# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SURF_MEASURES_METADATA = Metadata(
    id="21378ea7d8b0e920275d5e4fab477aa23c434ab4.boutiques",
    name="SurfMeasures",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class SurfMeasuresOutputs(typing.NamedTuple):
    """
    Output object returned when calling `surf_measures(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_1_d: OutputPathType | None
    """Output in 1D format"""
    output_dset: OutputPathType
    """Output in specified dataset format"""


def surf_measures(
    spec_file: InputPathType,
    surf_a: str,
    out_dset: str,
    surf_b: str | None = None,
    out_1_d: str | None = None,
    func: list[str] | None = None,
    surf_volume: InputPathType | None = None,
    cmask: str | None = None,
    debug: int | None = None,
    dnode: float | None = None,
    nodes_1_d: InputPathType | None = None,
    info_all: bool = False,
    info_area: bool = False,
    info_norms: bool = False,
    info_thick: bool = False,
    info_vol: bool = False,
    info_volg: bool = False,
    ver: bool = False,
    runner: Runner | None = None,
) -> SurfMeasuresOutputs:
    """
    Compute measures from surface dataset(s).
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        spec_file: SUMA spec file containing a list of related surfaces.
        surf_a: Surface name (in spec file) for the first surface.
        out_dset: Output filename with dataset format.
        surf_b: Surface name (in spec file) for the second surface.
        out_1_d: Output filename in 1D format.
        func: Measure function to be applied.
        surf_volume: AFNI volume dataset associated with the surface.
        cmask: Restrict nodes with a mask.
        debug: Display extra run-time information with specified debug level\
            (0-5).
        dnode: Display extra information for specified node.
        nodes_1_d: Restrict output to specific nodes listed in a file.
        info_all: Display all final info.
        info_area: Display total area of each triangulated surface.
        info_norms: Display info about the normals.
        info_thick: Display minimum and maximum thickness between surfaces.
        info_vol: Display total computed volume between surfaces.
        info_volg: Display total computed volume estimated via Gauss' theorem.
        ver: Show program version and compile date.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfMeasuresOutputs`).
    """
    if debug is not None and not (0 <= debug <= 5): 
        raise ValueError(f"'debug' must be between 0 <= x <= 5 but was {debug}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURF_MEASURES_METADATA)
    cargs = []
    cargs.append("SurfMeasures")
    cargs.extend([
        "-spec",
        execution.input_file(spec_file)
    ])
    cargs.extend([
        "-surf_A",
        surf_a
    ])
    if surf_b is not None:
        cargs.extend([
            "-surf_B",
            surf_b
        ])
    if out_1_d is not None:
        cargs.extend([
            "-out_1D",
            out_1_d
        ])
    cargs.extend([
        "-out",
        out_dset
    ])
    if func is not None:
        cargs.extend([
            "-func",
            *func
        ])
    if surf_volume is not None:
        cargs.extend([
            "-sv",
            execution.input_file(surf_volume)
        ])
    if cmask is not None:
        cargs.extend([
            "-cmask",
            cmask
        ])
    if debug is not None:
        cargs.extend([
            "-debug",
            str(debug)
        ])
    if dnode is not None:
        cargs.extend([
            "-dnode",
            str(dnode)
        ])
    if nodes_1_d is not None:
        cargs.extend([
            "-nodes_1D",
            execution.input_file(nodes_1_d)
        ])
    if info_all:
        cargs.append("-info_all")
    if info_area:
        cargs.append("-info_area")
    if info_norms:
        cargs.append("-info_norms")
    if info_thick:
        cargs.append("-info_thick")
    if info_vol:
        cargs.append("-info_vol")
    if info_volg:
        cargs.append("-info_volg")
    if ver:
        cargs.append("-ver")
    ret = SurfMeasuresOutputs(
        root=execution.output_file("."),
        output_1_d=execution.output_file(out_1_d + ".1D") if (out_1_d is not None) else None,
        output_dset=execution.output_file(out_dset),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SURF_MEASURES_METADATA",
    "SurfMeasuresOutputs",
    "surf_measures",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

UBER_SUBJECT_PY_METADATA = Metadata(
    id="d4f27b8c1311827036baefee6a5e5f22311c1cf3.boutiques",
    name="uber_subject.py",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class UberSubjectPyOutputs(typing.NamedTuple):
    """
    Output object returned when calling `uber_subject_py(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def uber_subject_py(
    qt_opts: str | None = None,
    svar: str | None = None,
    cvar: str | None = None,
    no_gui: bool = False,
    print_ap_command: bool = False,
    save_ap_command: str | None = None,
    exec_ap_command: bool = False,
    exec_proc_script: bool = False,
    align_cost: str | None = None,
    align_giant_move: str | None = None,
    align_opts_aea: str | None = None,
    anal_domain: str | None = None,
    anal_type: str | None = None,
    anat: InputPathType | None = None,
    anat_has_skull: str | None = None,
    blocks: str | None = None,
    blur_size: float | None = None,
    epi: str | None = None,
    epi_wildcard: str | None = None,
    gid: str | None = None,
    gltsym: str | None = None,
    gltsym_label: str | None = None,
    motion_limit: float | None = None,
    outlier_limit: float | None = None,
    regress_goforit: float | None = None,
    regress_bandpass: str | None = None,
    regress_jobs: float | None = None,
    regress_mot_deriv: str | None = None,
    regress_opts_3d_d: str | None = None,
    reml_exec: str | None = None,
    run_clustsim: str | None = None,
    sid: str | None = None,
    stim: InputPathType | None = None,
    stim_basis: str | None = None,
    stim_label: str | None = None,
    stim_type: str | None = None,
    stim_wildcard: str | None = None,
    tcat_nfirst: float | None = None,
    tlrc_base: str | None = None,
    tlrc_ok_maxite: str | None = None,
    tlrc_opts_at: str | None = None,
    volreg_base: str | None = None,
    verb: str | None = None,
    runner: Runner | None = None,
) -> UberSubjectPyOutputs:
    """
    Graphical interface to afni_proc.py.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        qt_opts: Pass options to PyQt4.
        svar: Set subject variable to value.
        cvar: Set control variable to value.
        no_gui: Do not open graphical interface.
        print_ap_command: Show afni_proc.py script.
        save_ap_command: Save afni_proc.py script.
        exec_ap_command: Run afni_proc.py command.
        exec_proc_script: Run proc script.
        align_cost: Specify cost function for anat/EPI alignment.
        align_giant_move: Use -giant_move in AEA.py.
        align_opts_aea: Specify extra options for align_epi_anat.py.
        anal_domain: Set data domain (volume/rest).
        anal_type: Set analysis type (task/rest).
        anat: Set anatomical dataset name.
        anat_has_skull: Whether anat has skull (yes/no).
        blocks: Set list of processing blocks to apply.
        blur_size: Set blur size in mm.
        epi: Set list of EPI datasets.
        epi_wildcard: Use wildcard for EPI datasets (yes/no).
        gid: Set group ID.
        gltsym: Specify list of symbolic GLTs.
        gltsym_label: Set corresponding GLT labels.
        motion_limit: Set per-TR motion limit in mm.
        outlier_limit: Specify outlier limit for censoring.
        regress_goforit: Set GOFORIT level in 3dDeconvolve.
        regress_bandpass: Specify bandpass limits to remain after regress.
        regress_jobs: Number of jobs to use in 3dDeconvolve.
        regress_mot_deriv: Regress motion derivatives (yes/no).
        regress_opts_3d_d: Specify extra options for 3dDeconvolve.
        reml_exec: Run 3dREMLfit (yes/no).
        run_clustsim: Run 3dClustSim (yes/no).
        sid: Set subject ID.
        stim: Set list of stim timing files.
        stim_basis: Set basis functions for stim classes.
        stim_label: Set stim file labels.
        stim_type: Set stim types for stim classes.
        stim_wildcard: Use wildcard for stim files (yes/no).
        tcat_nfirst: Set number of TRs to remove per run.
        tlrc_base: Specify anat for standard space alignment.
        tlrc_ok_maxite: Pass -OK_maxite to @auto_tlrc (yes/no).
        tlrc_opts_at: Specify extra options for @auto_tlrc.
        volreg_base: Set volreg base string (first/third/last).
        verb: Set verbose level.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `UberSubjectPyOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(UBER_SUBJECT_PY_METADATA)
    cargs = []
    cargs.append("uber_subject.py")
    if qt_opts is not None:
        cargs.extend([
            "-qt_opts",
            qt_opts
        ])
    if svar is not None:
        cargs.extend([
            "-svar",
            svar
        ])
    if cvar is not None:
        cargs.extend([
            "-cvar",
            cvar
        ])
    if no_gui:
        cargs.append("-no_gui")
    if print_ap_command:
        cargs.append("-print_ap_command")
    if save_ap_command is not None:
        cargs.extend([
            "-save_ap_command",
            save_ap_command
        ])
    if exec_ap_command:
        cargs.append("-exec_ap_command")
    if exec_proc_script:
        cargs.append("-exec_proc_script")
    if align_cost is not None:
        cargs.extend([
            "-align_cost",
            align_cost
        ])
    if align_giant_move is not None:
        cargs.extend([
            "-align_giant_move",
            align_giant_move
        ])
    if align_opts_aea is not None:
        cargs.extend([
            "-align_opts_aea",
            align_opts_aea
        ])
    if anal_domain is not None:
        cargs.extend([
            "-anal_domain",
            anal_domain
        ])
    if anal_type is not None:
        cargs.extend([
            "-anal_type",
            anal_type
        ])
    if anat is not None:
        cargs.extend([
            "-anat",
            execution.input_file(anat)
        ])
    if anat_has_skull is not None:
        cargs.extend([
            "-anat_has_skull",
            anat_has_skull
        ])
    if blocks is not None:
        cargs.extend([
            "-blocks",
            blocks
        ])
    if blur_size is not None:
        cargs.extend([
            "-blur_size",
            str(blur_size)
        ])
    if epi is not None:
        cargs.extend([
            "-epi",
            epi
        ])
    if epi_wildcard is not None:
        cargs.extend([
            "-epi_wildcard",
            epi_wildcard
        ])
    if gid is not None:
        cargs.extend([
            "-gid",
            gid
        ])
    if gltsym is not None:
        cargs.extend([
            "-gltsym",
            gltsym
        ])
    if gltsym_label is not None:
        cargs.extend([
            "-gltsym_label",
            gltsym_label
        ])
    if motion_limit is not None:
        cargs.extend([
            "-motion_limit",
            str(motion_limit)
        ])
    if outlier_limit is not None:
        cargs.extend([
            "-outlier_limit",
            str(outlier_limit)
        ])
    if regress_goforit is not None:
        cargs.extend([
            "-regress_GOFORIT",
            str(regress_goforit)
        ])
    if regress_bandpass is not None:
        cargs.extend([
            "-regress_bandpass",
            regress_bandpass
        ])
    if regress_jobs is not None:
        cargs.extend([
            "-regress_jobs",
            str(regress_jobs)
        ])
    if regress_mot_deriv is not None:
        cargs.extend([
            "-regress_mot_deriv",
            regress_mot_deriv
        ])
    if regress_opts_3d_d is not None:
        cargs.extend([
            "-regress_opts_3dD",
            regress_opts_3d_d
        ])
    if reml_exec is not None:
        cargs.extend([
            "-reml_exec",
            reml_exec
        ])
    if run_clustsim is not None:
        cargs.extend([
            "-run_clustsim",
            run_clustsim
        ])
    if sid is not None:
        cargs.extend([
            "-sid",
            sid
        ])
    if stim is not None:
        cargs.extend([
            "-stim",
            execution.input_file(stim)
        ])
    if stim_basis is not None:
        cargs.extend([
            "-stim_basis",
            stim_basis
        ])
    if stim_label is not None:
        cargs.extend([
            "-stim_label",
            stim_label
        ])
    if stim_type is not None:
        cargs.extend([
            "-stim_type",
            stim_type
        ])
    if stim_wildcard is not None:
        cargs.extend([
            "-stim_wildcard",
            stim_wildcard
        ])
    if tcat_nfirst is not None:
        cargs.extend([
            "-tcat_nfirst",
            str(tcat_nfirst)
        ])
    if tlrc_base is not None:
        cargs.extend([
            "-tlrc_base",
            tlrc_base
        ])
    if tlrc_ok_maxite is not None:
        cargs.extend([
            "-tlrc_ok_maxite",
            tlrc_ok_maxite
        ])
    if tlrc_opts_at is not None:
        cargs.extend([
            "-tlrc_opts_at",
            tlrc_opts_at
        ])
    if volreg_base is not None:
        cargs.extend([
            "-volreg_base",
            volreg_base
        ])
    if verb is not None:
        cargs.extend([
            "-verb",
            verb
        ])
    ret = UberSubjectPyOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "UBER_SUBJECT_PY_METADATA",
    "UberSubjectPyOutputs",
    "uber_subject_py",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

GIFTI_CONVERT_METADATA = Metadata(
    id="1119deb060240b3e26f7bbfea7cf6ac914e82758.boutiques",
    name="gifti-convert",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


class GiftiConvertOutputs(typing.NamedTuple):
    """
    Output object returned when calling `gifti_convert(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def gifti_convert(
    gifti_encoding: str,
    input_gifti_file: str,
    output_gifti_file: str,
    runner: Runner | None = None,
) -> GiftiConvertOutputs:
    """
    Convert a gifti file to a different encoding.
    
    The value of <gifti-encoding> must be one of the following:
    
    ASCII
    BASE64_BINARY
    GZIP_BASE64_BINARY
    EXTERNAL_FILE_BINARY.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        gifti_encoding: what the output encoding should be.
        input_gifti_file: the input gifti file.
        output_gifti_file: output - the output gifti file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `GiftiConvertOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(GIFTI_CONVERT_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-gifti-convert")
    cargs.append(gifti_encoding)
    cargs.append(input_gifti_file)
    cargs.append(output_gifti_file)
    ret = GiftiConvertOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "GIFTI_CONVERT_METADATA",
    "GiftiConvertOutputs",
    "gifti_convert",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

VOLUME_LABEL_PROBABILITY_METADATA = Metadata(
    id="0eb4968bb2e8b9aea5b76afc92179fafd1396a41.boutiques",
    name="volume-label-probability",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


class VolumeLabelProbabilityOutputs(typing.NamedTuple):
    """
    Output object returned when calling `volume_label_probability(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    probability_out: OutputPathType
    """the relative frequencies of each label at each voxel"""


def volume_label_probability(
    label_maps: InputPathType,
    probability_out: str,
    opt_exclude_unlabeled: bool = False,
    runner: Runner | None = None,
) -> VolumeLabelProbabilityOutputs:
    """
    Find frequency of volume labels.
    
    This command outputs a set of soft ROIs, one for each label in the input,
    where the value is how many of the input maps had that label at that voxel,
    divided by the number of input maps.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        label_maps: volume label file containing individual label maps from\
            many subjects.
        probability_out: the relative frequencies of each label at each voxel.
        opt_exclude_unlabeled: don't make a probability map of the unlabeled\
            key.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeLabelProbabilityOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_LABEL_PROBABILITY_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-volume-label-probability")
    cargs.append(execution.input_file(label_maps))
    cargs.append(probability_out)
    if opt_exclude_unlabeled:
        cargs.append("-exclude-unlabeled")
    ret = VolumeLabelProbabilityOutputs(
        root=execution.output_file("."),
        probability_out=execution.output_file(probability_out),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VOLUME_LABEL_PROBABILITY_METADATA",
    "VolumeLabelProbabilityOutputs",
    "volume_label_probability",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

RCA_FIX_ENTO_METADATA = Metadata(
    id="483fc4d892e00dbb3db293128c06ac20a3ba5034.boutiques",
    name="rca-fix-ento",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class RcaFixEntoOutputs(typing.NamedTuple):
    """
    Output object returned when calling `rca_fix_ento(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    entowm_output: OutputPathType
    """Output segmentation of the WM around entorhinal cortex."""
    finalsurfs_output: OutputPathType
    """Edited brain.finalsurfs with manual edits."""
    backup_finalsufs_output: OutputPathType
    """Backup of the original brain.finalsurfs.manedit.mgz before edits."""


def rca_fix_ento(
    subject: str,
    threads: float | None = None,
    submit: bool = False,
    account: str | None = None,
    brain_mask: bool = False,
    runner: Runner | None = None,
) -> RcaFixEntoOutputs:
    """
    A tool to fix the entorhinal white matter in FreeSurfer using a deep learning
    network.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subject: The subject identifier for processing.
        threads: Number of threads to use for processing.
        submit: Submit the task to sbatch with 1 thread and 14GB of memory.
        account: Specify the account to use when submitting to sbatch; default\
            is 'fhs'.
        brain_mask: Apply the ento fix to the brain.finalsurfs; this is turned\
            off due to a conflict with 255.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `RcaFixEntoOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(RCA_FIX_ENTO_METADATA)
    cargs = []
    cargs.append("rca-fix-ento")
    cargs.extend([
        "-s",
        subject
    ])
    if threads is not None:
        cargs.extend([
            "--threads",
            str(threads)
        ])
    if submit:
        cargs.append("--submit")
    if account is not None:
        cargs.extend([
            "--account",
            account
        ])
    if brain_mask:
        cargs.append("--brain-mask")
    ret = RcaFixEntoOutputs(
        root=execution.output_file("."),
        entowm_output=execution.output_file(subject + "/mri/entowm.mgz"),
        finalsurfs_output=execution.output_file(subject + "/mri/brain.finalsurfs.manedit.mgz"),
        backup_finalsufs_output=execution.output_file(subject + "/mri/backup.brain.finalsurfs.manedit.mgz"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "RCA_FIX_ENTO_METADATA",
    "RcaFixEntoOutputs",
    "rca_fix_ento",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MIDEFACE_METADATA = Metadata(
    id="38b6a923ce22638bd0e6fc77f68e3ac76360f9d7.boutiques",
    name="mideface",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MidefaceOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mideface(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    defaced_output: OutputPathType
    """Defaced output volume"""
    facemask_output: OutputPathType | None
    """Applied facemask file"""


def mideface(
    input_volume: InputPathType,
    output_volume: InputPathType,
    facemask: InputPathType | None = None,
    output_dir: str | None = None,
    exclusion_mask: InputPathType | None = None,
    samseg_ndilations: float | None = None,
    samseg_json: str | None = None,
    init_reg: InputPathType | None = None,
    synthseg_ndilations: float | None = None,
    fill_const: list[float] | None = None,
    fhi: float | None = None,
    code_: str | None = None,
    image_convert: str | None = None,
    threads: float | None = None,
    display_no: float | None = None,
    apply_volume: str | None = None,
    check_volume: InputPathType | None = None,
    check_output_file: InputPathType | None = None,
    output_format: str | None = None,
    runner: Runner | None = None,
) -> MidefaceOutputs:
    """
    Minimally invasive defacing tool.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_volume: Volume to deface.
        output_volume: Defaced output volume.
        facemask: Facemask to apply.
        output_dir: Directory for outputs, activates PostHeadSurf.
        exclusion_mask: Mask to exclude certain regions from defacing.
        samseg_ndilations: Number of dilations for Samseg segmentation.
        samseg_json: JSON configuration for Samseg.
        init_reg: Initial registration file for Samseg.
        synthseg_ndilations: Number of dilations for Synthseg segmentation.
        fill_const: Constants for filling regions.
        fhi: FHI value for MRIchangeType().
        code_: Embed code name in pictures.
        image_convert: Path to ImageMagick convert binary for pictures.
        threads: Number of threads to use.
        display_no: Xvfb display number for taking pictures.
        apply_volume: Apply midface output to a second volume.
        check_volume: Volume to check if defaced.
        check_output_file: Optional output file for check result.
        output_format: Output file format.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MidefaceOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MIDEFACE_METADATA)
    cargs = []
    cargs.append("mideface")
    cargs.extend([
        "--i",
        execution.input_file(input_volume)
    ])
    cargs.extend([
        "--o",
        execution.input_file(output_volume)
    ])
    if facemask is not None:
        cargs.extend([
            "--facemask",
            execution.input_file(facemask)
        ])
    if output_dir is not None:
        cargs.extend([
            "--odir",
            output_dir
        ])
    if exclusion_mask is not None:
        cargs.extend([
            "--xmask",
            execution.input_file(exclusion_mask)
        ])
    if samseg_ndilations is not None:
        cargs.extend([
            "--xmask-samseg",
            str(samseg_ndilations)
        ])
    if samseg_json is not None:
        cargs.extend([
            "--samseg-json",
            samseg_json
        ])
    if init_reg is not None:
        cargs.extend([
            "--init-reg",
            execution.input_file(init_reg)
        ])
    if synthseg_ndilations is not None:
        cargs.extend([
            "--xmask-synthseg",
            str(synthseg_ndilations)
        ])
    if fill_const is not None:
        cargs.extend([
            "--fill-const",
            *map(str, fill_const)
        ])
    if fhi is not None:
        cargs.extend([
            "--fhi",
            str(fhi)
        ])
    if code_ is not None:
        cargs.extend([
            "--code",
            code_
        ])
    if image_convert is not None:
        cargs.extend([
            "--imconvert",
            image_convert
        ])
    if threads is not None:
        cargs.extend([
            "--threads",
            str(threads)
        ])
    if display_no is not None:
        cargs.extend([
            "--display",
            str(display_no)
        ])
    if apply_volume is not None:
        cargs.extend([
            "--apply",
            apply_volume
        ])
    if check_volume is not None:
        cargs.extend([
            "--check",
            execution.input_file(check_volume)
        ])
    if check_output_file is not None:
        cargs.extend([
            "--check",
            execution.input_file(check_output_file)
        ])
    if output_format is not None:
        cargs.extend([
            "--nii --nii.gz --mgz",
            output_format
        ])
    ret = MidefaceOutputs(
        root=execution.output_file("."),
        defaced_output=execution.output_file(pathlib.Path(output_volume).name),
        facemask_output=execution.output_file(pathlib.Path(facemask).name) if (facemask is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MIDEFACE_METADATA",
    "MidefaceOutputs",
    "mideface",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SEGMENT_THALAMIC_NUCLEI_SH_METADATA = Metadata(
    id="8160107af015d05eab1a7764f08870c3a3705de2.boutiques",
    name="segmentThalamicNuclei.sh",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class SegmentThalamicNucleiShOutputs(typing.NamedTuple):
    """
    Output object returned when calling `segment_thalamic_nuclei_sh(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    thalamic_nuclei_output: OutputPathType
    """Segmented thalamic nuclei output file"""


def segment_thalamic_nuclei_sh(
    subject_id: str,
    output_dir: str,
    runner: Runner | None = None,
) -> SegmentThalamicNucleiShOutputs:
    """
    Segment the thalamic nuclei using FreeSurfer.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subject_id: Subject ID for segmentation.
        output_dir: Output directory for the segmented thalamic nuclei.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SegmentThalamicNucleiShOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SEGMENT_THALAMIC_NUCLEI_SH_METADATA)
    cargs = []
    cargs.append("segmentThalamicNuclei.sh")
    cargs.append(subject_id)
    cargs.append(output_dir)
    ret = SegmentThalamicNucleiShOutputs(
        root=execution.output_file("."),
        thalamic_nuclei_output=execution.output_file(output_dir + "/" + subject_id + "_thalamic_nuclei.mgz"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SEGMENT_THALAMIC_NUCLEI_SH_METADATA",
    "SegmentThalamicNucleiShOutputs",
    "segment_thalamic_nuclei_sh",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FOCI_CREATE_METADATA = Metadata(
    id="f54b92ca66ce06ab8acae5b43f6f0c242d214886.boutiques",
    name="foci-create",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


@dataclasses.dataclass
class FociCreateClass:
    """
    specify class input data.
    """
    class_name: str
    """name of class"""
    foci_list_file: str
    """text file containing foci names, coordinates, and colors"""
    surface: InputPathType
    """surface file for projection of foci list file"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-class")
        cargs.append(self.class_name)
        cargs.append(self.foci_list_file)
        cargs.append(execution.input_file(self.surface))
        return cargs


class FociCreateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `foci_create(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output foci file"""


def foci_create(
    output: str,
    class_: list[FociCreateClass] | None = None,
    runner: Runner | None = None,
) -> FociCreateOutputs:
    """
    Create a foci file.
    
    Creates a foci file from names, coordinates, and RGB values in a text file.
    The text file must have the following format (2 lines per focus):
    
    <focus-name>
    <red> <green> <blue> <x> <y> <z>
    ...
    
    Foci names are specified on a separate line from their coordinates and
    color, in order to let foci names contain spaces. Whitespace is trimmed from
    both ends of the foci name, but is kept if it is in the middle of a name.
    The values of <red>, <green>, <blue> and must be integers from 0 to 255, and
    will specify the color the foci is drawn as.
    
    Foci are grouped into classes and the name for the class is specified using
    the <class-name> parameter.
    
    All foci within one text file must be associated with the structure
    contained in the <surface> parameter and are projected to that surface.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        output: the output foci file.
        class_: specify class input data.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FociCreateOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FOCI_CREATE_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-foci-create")
    cargs.append(output)
    if class_ is not None:
        cargs.extend([a for c in [s.run(execution) for s in class_] for a in c])
    ret = FociCreateOutputs(
        root=execution.output_file("."),
        output=execution.output_file(output),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FOCI_CREATE_METADATA",
    "FociCreateClass",
    "FociCreateOutputs",
    "foci_create",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FIXEL2SH_METADATA = Metadata(
    id="ad311f3879760b3a8467869800c42a82cc4da4d8.boutiques",
    name="fixel2sh",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


@dataclasses.dataclass
class Fixel2shConfig:
    """
    temporarily set the value of an MRtrix config file entry.
    """
    key: str
    """temporarily set the value of an MRtrix config file entry."""
    value: str
    """temporarily set the value of an MRtrix config file entry."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-config")
        cargs.append(self.key)
        cargs.append(self.value)
        return cargs


class Fixel2shOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fixel2sh(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    sh_out: OutputPathType
    """the output sh image."""


def fixel2sh(
    fixel_in: InputPathType,
    sh_out: str,
    lmax: int | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Fixel2shConfig] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Fixel2shOutputs:
    """
    Convert a fixel-based sparse-data image into an spherical harmonic image.
    
    This command generates spherical harmonic data from fixels that can be
    visualised using the ODF tool in MRview. The output ODF lobes are scaled
    according to the values in the input fixel image.
    
    The spherical harmonic coefficients are stored according the conventions
    described the main documentation, which can be found at the following link:
    https://mrtrix.readthedocs.io/en/3.0.4/concepts/spherical_harmonics.html
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        fixel_in: the input fixel data file.
        sh_out: the output sh image.
        lmax: set the maximum harmonic order for the output series (Default: 8).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Fixel2shOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FIXEL2SH_METADATA)
    cargs = []
    cargs.append("fixel2sh")
    if lmax is not None:
        cargs.extend([
            "-lmax",
            str(lmax)
        ])
    if info:
        cargs.append("-info")
    if quiet:
        cargs.append("-quiet")
    if debug:
        cargs.append("-debug")
    if force:
        cargs.append("-force")
    if nthreads is not None:
        cargs.extend([
            "-nthreads",
            str(nthreads)
        ])
    if config is not None:
        cargs.extend([a for c in [s.run(execution) for s in config] for a in c])
    if help_:
        cargs.append("-help")
    if version:
        cargs.append("-version")
    cargs.append(execution.input_file(fixel_in))
    cargs.append(sh_out)
    ret = Fixel2shOutputs(
        root=execution.output_file("."),
        sh_out=execution.output_file(sh_out),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FIXEL2SH_METADATA",
    "Fixel2shConfig",
    "Fixel2shOutputs",
    "fixel2sh",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FOD2DEC_METADATA = Metadata(
    id="464092d0c46e0f6d44851c80e38f43548c357c00.boutiques",
    name="fod2dec",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


@dataclasses.dataclass
class Fod2decConfig:
    """
    temporarily set the value of an MRtrix config file entry.
    """
    key: str
    """temporarily set the value of an MRtrix config file entry."""
    value: str
    """temporarily set the value of an MRtrix config file entry."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-config")
        cargs.append(self.key)
        cargs.append(self.value)
        return cargs


class Fod2decOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fod2dec(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """The output DEC image (weighted RGB triplets)."""


def fod2dec(
    input_: InputPathType,
    output: str,
    mask: InputPathType | None = None,
    contrast: InputPathType | None = None,
    lum: bool = False,
    lum_coefs: list[float] | None = None,
    lum_gamma: float | None = None,
    threshold: float | None = None,
    no_weight: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Fod2decConfig] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Fod2decOutputs:
    """
    Generate FOD-based DEC maps, with optional panchromatic sharpening and/or
    luminance/perception correction.
    
    By default, the FOD-based DEC is weighted by the integral of the FOD. To
    weight by another scalar map, use the -contrast option. This option can also
    be used for panchromatic sharpening, e.g., by supplying a T1 (or other
    sensible) anatomical volume with a higher spatial resolution.
    
    References:
    
    Dhollander T, Smith RE, Tournier JD, Jeurissen B, Connelly A. Time to move
    on: an FOD-based DEC map to replace DTI's trademark DEC FA. Proc Intl Soc
    Mag Reson Med, 2015, 23, 1027
    
    Dhollander T, Raffelt D, Smith RE, Connelly A. Panchromatic sharpening of
    FOD-based DEC maps by structural T1 information. Proc Intl Soc Mag Reson
    Med, 2015, 23, 566.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_: The input FOD image (spherical harmonic coefficients).
        output: The output DEC image (weighted RGB triplets).
        mask: Only perform DEC computation within the specified mask image.
        contrast: Weight the computed DEC map by the provided image contrast.\
            If the contrast has a different image grid, the DEC map is first\
            resliced and renormalised. To achieve panchromatic sharpening, provide\
            an image with a higher spatial resolution than the input FOD image;\
            e.g., a T1 anatomical volume. Only the DEC is subject to the mask, so\
            as to allow for partial colouring of the contrast image.\
            Default when this option is *not* provided: integral of input FOD,\
            subject to the same mask/threshold as used for DEC computation.
        lum: Correct for luminance/perception, using default values Cr,Cg,Cb =\
            0.3,0.5,0.2 and gamma = 2.2 (*not* correcting is the theoretical\
            equivalent of Cr,Cg,Cb = 1,1,1 and gamma = 2).
        lum_coefs: The coefficients Cr,Cg,Cb to correct for\
            luminance/perception.\
            Note: this implicitly switches on luminance/perception correction,\
            using a default gamma = 2.2 unless specified otherwise.
        lum_gamma: The gamma value to correct for luminance/perception.\
            Note: this implicitly switches on luminance/perception correction,\
            using a default Cr,Cg,Cb = 0.3,0.5,0.2 unless specified otherwise.
        threshold: FOD amplitudes below the threshold value are considered\
            zero.
        no_weight: Do not weight the DEC map; just output the unweighted\
            colours. Reslicing and renormalising of colours will still happen when\
            providing the -contrast option as a template.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Fod2decOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FOD2DEC_METADATA)
    cargs = []
    cargs.append("fod2dec")
    if mask is not None:
        cargs.extend([
            "-mask",
            execution.input_file(mask)
        ])
    if contrast is not None:
        cargs.extend([
            "-contrast",
            execution.input_file(contrast)
        ])
    if lum:
        cargs.append("-lum")
    if lum_coefs is not None:
        cargs.extend([
            "-lum_coefs",
            *map(str, lum_coefs)
        ])
    if lum_gamma is not None:
        cargs.extend([
            "-lum_gamma",
            str(lum_gamma)
        ])
    if threshold is not None:
        cargs.extend([
            "-threshold",
            str(threshold)
        ])
    if no_weight:
        cargs.append("-no_weight")
    if info:
        cargs.append("-info")
    if quiet:
        cargs.append("-quiet")
    if debug:
        cargs.append("-debug")
    if force:
        cargs.append("-force")
    if nthreads is not None:
        cargs.extend([
            "-nthreads",
            str(nthreads)
        ])
    if config is not None:
        cargs.extend([a for c in [s.run(execution) for s in config] for a in c])
    if help_:
        cargs.append("-help")
    if version:
        cargs.append("-version")
    cargs.append(execution.input_file(input_))
    cargs.append(output)
    ret = Fod2decOutputs(
        root=execution.output_file("."),
        output=execution.output_file(output),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FOD2DEC_METADATA",
    "Fod2decConfig",
    "Fod2decOutputs",
    "fod2dec",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

REINFLATE_SUBJECT_METADATA = Metadata(
    id="1e08a83747d403bdb6fd509a84d2ca27a452f873.boutiques",
    name="reinflate_subject",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class ReinflateSubjectOutputs(typing.NamedTuple):
    """
    Output object returned when calling `reinflate_subject(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def reinflate_subject(
    args: str | None = None,
    runner: Runner | None = None,
) -> ReinflateSubjectOutputs:
    """
    Tool for reinflating brain surfaces.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        args: Arguments for reinflate_subject.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ReinflateSubjectOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(REINFLATE_SUBJECT_METADATA)
    cargs = []
    cargs.append("reinflate_subject")
    if args is not None:
        cargs.append(args)
    ret = ReinflateSubjectOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "REINFLATE_SUBJECT_METADATA",
    "ReinflateSubjectOutputs",
    "reinflate_subject",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

AFNI_METADATA = Metadata(
    id="40dab3442f2e18227072a94a9827898d65dc4372.boutiques",
    name="afni",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class AfniOutputs(typing.NamedTuple):
    """
    Output object returned when calling `afni_(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    session_output: OutputPathType
    """Output file for the session data"""


def afni_(
    session_directories: str | None = None,
    bysub: list[str] | None = None,
    all_dsets: bool = False,
    purge: bool = False,
    posfunc: bool = False,
    recursive: bool = False,
    no1_d: bool = False,
    nocsv: bool = False,
    notsv: bool = False,
    unique: bool = False,
    orient: str | None = None,
    noplugins: bool = False,
    seehidden: bool = False,
    allow_all_plugins: bool = False,
    yesplugouts: bool = False,
    debug_plugouts: bool = False,
    noplugouts: bool = False,
    skip_afnirc: bool = False,
    layout: InputPathType | None = None,
    niml: bool = False,
    np: int | None = None,
    npq: int | None = None,
    npb: int | None = None,
    com: str | None = None,
    comsep: str | None = None,
    runner: Runner | None = None,
) -> AfniOutputs:
    """
    Tool for reading in sessions of 3D datasets and visualizing them.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        session_directories: Input session directories containing the datasets.
        bysub: Gather all datasets corresponding to a single subject identifier.
        all_dsets: Read in all datasets from all listed folders together.
        purge: Conserve memory by purging unused datasets.
        posfunc: Start up the color 'pbar' to use only positive function values.
        recursive: Recursively search each session_directory for more session\
            subdirectories.
        no1_d: Tells AFNI not to read *.1D timeseries files.
        nocsv: Tells AFNI not to read *.csv files.
        notsv: Tells AFNI not to read *.tsv files.
        unique: Create a unique set of colors for each AFNI controller window.
        orient: Orientation code for displaying x-y-z coordinates.
        noplugins: Do not load plugins.
        seehidden: Show hidden plugins.
        allow_all_plugins: Do not hide plugins.
        yesplugouts: Listen for plugouts.
        debug_plugouts: Plugout code prints lots of messages (for debugging).
        noplugouts: Do not listen for plugouts.
        skip_afnirc: Do not read .afnirc file.
        layout: Read initial windows layout from a file.
        niml: Turn on listening for NIML-formatted data from SUMA.
        np: Provide a port offset for multiple instances.
        npq: Like -np but quieter in case of errors.
        npb: Provide a block of port numbers.
        com: Specify command strings to drive AFNI on startup.
        comsep: Character to use as a separator for commands.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AfniOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(AFNI_METADATA)
    cargs = []
    cargs.append("afni")
    if session_directories is not None:
        cargs.append(session_directories)
    if bysub is not None:
        cargs.extend([
            "-bysub",
            *bysub
        ])
    if all_dsets:
        cargs.append("-all_dsets")
    if purge:
        cargs.append("-purge")
    if posfunc:
        cargs.append("-posfunc")
    if recursive:
        cargs.append("-R")
    if no1_d:
        cargs.append("-no1D")
    if nocsv:
        cargs.append("-nocsv")
    if notsv:
        cargs.append("-notsv")
    if unique:
        cargs.append("-unique")
    if orient is not None:
        cargs.extend([
            "-orient",
            orient
        ])
    if noplugins:
        cargs.append("-noplugins")
    if seehidden:
        cargs.append("-seehidden")
    if allow_all_plugins:
        cargs.append("-DAFNI_ALLOW_ALL_PLUGINS=YES")
    if yesplugouts:
        cargs.append("-yesplugouts")
    if debug_plugouts:
        cargs.append("-YESplugouts")
    if noplugouts:
        cargs.append("-noplugouts")
    if skip_afnirc:
        cargs.append("-skip_afnirc")
    if layout is not None:
        cargs.extend([
            "-layout",
            execution.input_file(layout)
        ])
    if niml:
        cargs.append("-niml")
    if np is not None:
        cargs.extend([
            "-np",
            str(np)
        ])
    if npq is not None:
        cargs.extend([
            "-npq",
            str(npq)
        ])
    if npb is not None:
        cargs.extend([
            "-npb",
            str(npb)
        ])
    if com is not None:
        cargs.extend([
            "-com",
            com
        ])
    if comsep is not None:
        cargs.extend([
            "-comsep",
            comsep
        ])
    ret = AfniOutputs(
        root=execution.output_file("."),
        session_output=execution.output_file("output_session.nii.gz"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "AFNI_METADATA",
    "AfniOutputs",
    "afni_",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_GLMFIT_METADATA = Metadata(
    id="79f830582903b4c568e10c98abbe12709997f2f4.boutiques",
    name="mri_glmfit",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriGlmfitOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_glmfit(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    regression_coefs: OutputPathType
    """All regression coefficients (B)."""
    residual_error: OutputPathType
    """Residual error."""
    residual_variance: OutputPathType
    """Residual error variance."""
    residual_stddev: OutputPathType
    """Residual error standard deviation."""
    fsgd_output: OutputPathType
    """FSGD file if one was input."""
    normalized_weights: OutputPathType
    """Normalized weights with --w or --wls."""
    signal_estimate: OutputPathType
    """Signal estimate with --save-yhat."""
    final_mask: OutputPathType
    """Final mask when a mask is used."""
    design_condition: OutputPathType
    """Design matrix condition at each voxel with --save-cond."""
    contrast_output_dir: OutputPathType
    """Directory for each contrast output files."""


def mri_glmfit(
    y_input: InputPathType,
    glmdir: str | None = None,
    table_input: InputPathType | None = None,
    fsgd: InputPathType | None = None,
    design_matrix: InputPathType | None = None,
    contrast_matrix: list[InputPathType] | None = None,
    osgm_flag: bool = False,
    no_contrasts_ok_flag: bool = False,
    dti_params: list[str] | None = None,
    dti_matrix: InputPathType | None = None,
    pvr: list[InputPathType] | None = None,
    selfreg: list[float] | None = None,
    wls: str | None = None,
    yffxvar: InputPathType | None = None,
    ffxdof: float | None = None,
    ffxdofdat: InputPathType | None = None,
    weight: InputPathType | None = None,
    weight_inv_flag: bool = False,
    weight_sqrt_flag: bool = False,
    fwhm: float | None = None,
    var_fwhm: float | None = None,
    no_mask_smooth_flag: bool = False,
    no_est_fwhm_flag: bool = False,
    mask: InputPathType | None = None,
    label: InputPathType | None = None,
    no_mask_flag: bool = False,
    no_cortex_flag: bool = False,
    mask_inv_flag: bool = False,
    prune_flag: bool = False,
    no_prune_flag: bool = False,
    logy_flag: bool = False,
    no_logy_flag: bool = False,
    rm_spatial_mean_flag: bool = False,
    yhat_save_flag: bool = False,
    eres_save_flag: bool = False,
    eres_scm_flag: bool = False,
    save_fwhm_map_flag: bool = False,
    y_out: InputPathType | None = None,
    surface: str | None = None,
    skew_flag: bool = False,
    kurtosis_flag: bool = False,
    sim_params: list[str] | None = None,
    sim_sign: str | None = None,
    uniform_params: list[str] | None = None,
    permute_input_flag: bool = False,
    pca_flag: bool = False,
    tar1_flag: bool = False,
    save_yhat_flag: bool = False,
    save_cond_flag: bool = False,
    voxdump: list[float] | None = None,
    seed: float | None = None,
    synth_flag: bool = False,
    resynthtest_it: float | None = None,
    profile_it: float | None = None,
    mrtm1_params: list[str] | None = None,
    mrtm2_params: list[str] | None = None,
    logan_params: list[str] | None = None,
    bp_clip_neg_flag: bool = False,
    bp_clip_max: float | None = None,
    perm_force_flag: bool = False,
    diag_level: float | None = None,
    diag_cluster_flag: bool = False,
    debug_flag: bool = False,
    checkopts_flag: bool = False,
    help_flag: bool = False,
    version_flag: bool = False,
    no_fix_vertex_area_flag: bool = False,
    allowsubjrep_flag: bool = False,
    allow_zero_dof_flag: bool = False,
    illcond_flag: bool = False,
    sim_done_file: InputPathType | None = None,
    no_sig_double_flag: bool = False,
    runner: Runner | None = None,
) -> MriGlmfitOutputs:
    """
    Performs general linear model (GLM) analysis in the volume or the surface.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        y_input: Path to input file where each frame is a separate input.\
            Accepts volume or surface-based formats.
        glmdir: Directory where output will be saved.
        table_input: Use text table as input instead of --y. Table should be of\
            form produced by asegstats2table or aparcstats2table.
        fsgd: Specify the global design matrix with a FreeSurfer Group\
            Descriptor File (FSGDF).
        design_matrix: Explicitly specify the design matrix. Can be in simple\
            text or in matlab4 format.
        contrast_matrix: Specify one or more contrasts to test. File should be\
            ASCII text with the contrast matrix.
        osgm_flag: Construct X and C as a one-sample group mean.
        no_contrasts_ok_flag: Do not fail if no contrasts are specified.
        dti_params: Do DTI analysis using bvals and bvecs.
        dti_matrix: Do DTI analysis using provided matrix.
        pvr: Per-voxel (or vertex) regressors.
        selfreg: Create a 'self-regressor' from the input data based on the\
            waveform at index col row slice.
        wls: Perform weighted least squares (WLS) random effects analysis\
            instead of ordinary least squares (OLS) using yffxvar.
        yffxvar: For fixed effects analysis.
        ffxdof: Degrees of Freedom (DOF) for fixed effects analysis.
        ffxdofdat: Text file with DOF for fixed effects analysis.
        weight: Perform weighted LMS using per-voxel weights from the\
            weightfile.
        weight_inv_flag: Invert weights.
        weight_sqrt_flag: Square root of (inverted) weights.
        fwhm: Smooth input with a Gaussian kernel, specified in mm.
        var_fwhm: Smooth residual variance map with a Gaussian kernel,\
            specified in mm.
        no_mask_smooth_flag: Do not mask when smoothing.
        no_est_fwhm_flag: Turn off FWHM output estimation.
        mask: Binary mask file for analysis.
        label: Use label as mask for surface data.
        no_mask_flag: Do NOT use a mask, same as --no-cortex.
        no_cortex_flag: Do NOT use subjects ?h.cortex.label as --label.
        mask_inv_flag: Invert mask for analysis.
        prune_flag: Remove voxels that do not have a non-zero value at each\
            frame.
        no_prune_flag: Do not prune zero-value voxels.
        logy_flag: Compute natural log of y prior to analysis.
        no_logy_flag: Do not compute natural log of y prior to analysis.
        rm_spatial_mean_flag: Subtract the (masked) mean from each frame.
        yhat_save_flag: Save signal estimate (yhat).
        eres_save_flag: Save residual error (eres).
        eres_scm_flag: Save residual error spatial correlation matrix\
            (eres.scm).
        save_fwhm_map_flag: Save voxel-wise map of FWHM estimates.
        y_out: Save input after any pre-processing.
        surface: Specify that the input has a surface geometry from the\
            hemisphere of the given FreeSurfer subject. Required for surface data\
            operations.
        skew_flag: Compute skew and p-value for skew.
        kurtosis_flag: Compute kurtosis and p-value for kurtosis.
        sim_params: Simulate data for statistical testing. Specify nulltype,\
            number of simulations, threshold and csd basename.
        sim_sign: Specify sign for simulation: abs, pos, or neg.
        uniform_params: Use uniform distribution for mc-full, specify min and\
            max.
        permute_input_flag: Permute input for testing purposes (not related to\
            simulation).
        pca_flag: Perform PCA/SVD analysis on the residual.
        tar1_flag: Compute and save temporal AR1 of residual.
        save_yhat_flag: Flag to save the signal estimate.
        save_cond_flag: Flag to save design matrix condition at each voxel.
        voxdump: Save GLM data for a single voxel at the specified col, row,\
            slice.
        seed: Use seed for random number generation.
        synth_flag: Replace input data with Gaussian noise for testing.
        resynthtest_it: Test GLM by resynthsis with the number of iterations\
            specified.
        profile_it: Test speed with specified number of iterations.
        mrtm1_params: Perform MRTM1 kinetic modeling with specified reference\
            tissue activity and time in seconds.
        mrtm2_params: Perform MRTM2 kinetic modeling with specified parameters.
        logan_params: Perform Logan kinetic modeling with specified parameters.
        bp_clip_neg_flag: Set negative BP voxels to 0.
        bp_clip_max: Set BP voxels above max to max.
        perm_force_flag: Force permutation test, even when design matrix is not\
            orthogonal.
        diag_level: Set diagnostic level.
        diag_cluster_flag: Save significant volume and exit from first\
            simulation loop.
        debug_flag: Turn on debugging mode.
        checkopts_flag: Check options and exit without executing.
        help_flag: Display help information.
        version_flag: Print out version and exit.
        no_fix_vertex_area_flag: Turn off fixing of vertex area (backward\
            compatibility).
        allowsubjrep_flag: Allow subject names to repeat in the fsgd file.
        allow_zero_dof_flag: Allow analyses with zero degrees of freedom.
        illcond_flag: Allow ill-conditioned design matrices.
        sim_done_file: Create done file when simulation finishes.
        no_sig_double_flag: Compute sig = -log10(p) from float p value, not\
            double.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriGlmfitOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_GLMFIT_METADATA)
    cargs = []
    cargs.append("mri_glmfit")
    if glmdir is not None:
        cargs.extend([
            "--glmdir",
            glmdir
        ])
    cargs.extend([
        "--y",
        execution.input_file(y_input)
    ])
    if table_input is not None:
        cargs.extend([
            "--table",
            execution.input_file(table_input)
        ])
    if fsgd is not None:
        cargs.extend([
            "--fsgd",
            execution.input_file(fsgd)
        ])
    if design_matrix is not None:
        cargs.extend([
            "--X",
            execution.input_file(design_matrix)
        ])
    if contrast_matrix is not None:
        cargs.extend([
            "--C",
            *[execution.input_file(f) for f in contrast_matrix]
        ])
    if osgm_flag:
        cargs.append("--osgm")
    if no_contrasts_ok_flag:
        cargs.append("--no-contrasts-ok")
    if dti_params is not None:
        cargs.extend([
            "--dti",
            *dti_params
        ])
    if dti_matrix is not None:
        cargs.extend([
            "--dti-X",
            execution.input_file(dti_matrix)
        ])
    if pvr is not None:
        cargs.extend([
            "--pvr",
            *[execution.input_file(f) for f in pvr]
        ])
    if selfreg is not None:
        cargs.extend([
            "--selfreg",
            *map(str, selfreg)
        ])
    if wls is not None:
        cargs.extend([
            "--wls",
            wls
        ])
    if yffxvar is not None:
        cargs.extend([
            "--yffxvar",
            execution.input_file(yffxvar)
        ])
    if ffxdof is not None:
        cargs.extend([
            "--ffxdof",
            str(ffxdof)
        ])
    if ffxdofdat is not None:
        cargs.extend([
            "--ffxdofdat",
            execution.input_file(ffxdofdat)
        ])
    if weight is not None:
        cargs.extend([
            "--w",
            execution.input_file(weight)
        ])
    if weight_inv_flag:
        cargs.append("--w-inv")
    if weight_sqrt_flag:
        cargs.append("--w-sqrt")
    if fwhm is not None:
        cargs.extend([
            "--fwhm",
            str(fwhm)
        ])
    if var_fwhm is not None:
        cargs.extend([
            "--var-fwhm",
            str(var_fwhm)
        ])
    if no_mask_smooth_flag:
        cargs.append("--no-mask-smooth")
    if no_est_fwhm_flag:
        cargs.append("--no-est-fwhm")
    if mask is not None:
        cargs.extend([
            "--mask",
            execution.input_file(mask)
        ])
    if label is not None:
        cargs.extend([
            "--label",
            execution.input_file(label)
        ])
    if no_mask_flag:
        cargs.append("--no-mask")
    if no_cortex_flag:
        cargs.append("--no-cortex")
    if mask_inv_flag:
        cargs.append("--mask-inv")
    if prune_flag:
        cargs.append("--prune")
    if no_prune_flag:
        cargs.append("--no-prune")
    if logy_flag:
        cargs.append("--logy")
    if no_logy_flag:
        cargs.append("--no-logy")
    if rm_spatial_mean_flag:
        cargs.append("--rm-spatial-mean")
    if yhat_save_flag:
        cargs.append("--yhat-save")
    if eres_save_flag:
        cargs.append("--eres-save")
    if eres_scm_flag:
        cargs.append("--eres-scm")
    if save_fwhm_map_flag:
        cargs.append("--save-fwhm-map")
    if y_out is not None:
        cargs.extend([
            "--y-out",
            execution.input_file(y_out)
        ])
    if surface is not None:
        cargs.extend([
            "--surf",
            surface
        ])
    if skew_flag:
        cargs.append("--skew")
    if kurtosis_flag:
        cargs.append("--kurtosis")
    if sim_params is not None:
        cargs.extend([
            "--sim",
            *sim_params
        ])
    if sim_sign is not None:
        cargs.extend([
            "--sim-sign",
            sim_sign
        ])
    if uniform_params is not None:
        cargs.extend([
            "--uniform",
            *uniform_params
        ])
    if permute_input_flag:
        cargs.append("--permute-input")
    if pca_flag:
        cargs.append("--pca")
    if tar1_flag:
        cargs.append("--tar1")
    if save_yhat_flag:
        cargs.append("--save-yhat")
    if save_cond_flag:
        cargs.append("--save-cond")
    if voxdump is not None:
        cargs.extend([
            "--voxdump",
            *map(str, voxdump)
        ])
    if seed is not None:
        cargs.extend([
            "--seed",
            str(seed)
        ])
    if synth_flag:
        cargs.append("--synth")
    if resynthtest_it is not None:
        cargs.extend([
            "--resynthtest",
            str(resynthtest_it)
        ])
    if profile_it is not None:
        cargs.extend([
            "--profile",
            str(profile_it)
        ])
    if mrtm1_params is not None:
        cargs.extend([
            "--mrtm1",
            *mrtm1_params
        ])
    if mrtm2_params is not None:
        cargs.extend([
            "--mrtm2",
            *mrtm2_params
        ])
    if logan_params is not None:
        cargs.extend([
            "--logan",
            *logan_params
        ])
    if bp_clip_neg_flag:
        cargs.append("--bp-clip-neg")
    if bp_clip_max is not None:
        cargs.extend([
            "--bp-clip-max",
            str(bp_clip_max)
        ])
    if perm_force_flag:
        cargs.append("--perm-force")
    if diag_level is not None:
        cargs.extend([
            "--diag",
            str(diag_level)
        ])
    if diag_cluster_flag:
        cargs.append("--diag-cluster")
    if debug_flag:
        cargs.append("--debug")
    if checkopts_flag:
        cargs.append("--checkopts")
    if help_flag:
        cargs.append("--help")
    if version_flag:
        cargs.append("--version")
    if no_fix_vertex_area_flag:
        cargs.append("--no-fix-vertex-area")
    if allowsubjrep_flag:
        cargs.append("--allowsubjrep")
    if allow_zero_dof_flag:
        cargs.append("--allow-zero-dof")
    if illcond_flag:
        cargs.append("--illcond")
    if sim_done_file is not None:
        cargs.extend([
            "--sim-done",
            execution.input_file(sim_done_file)
        ])
    if no_sig_double_flag:
        cargs.append("--no-sig-double")
    ret = MriGlmfitOutputs(
        root=execution.output_file("."),
        regression_coefs=execution.output_file("beta.mgh"),
        residual_error=execution.output_file("eres.mgh"),
        residual_variance=execution.output_file("rvar.mgh"),
        residual_stddev=execution.output_file("rstd.mgh"),
        fsgd_output=execution.output_file("y.fsgd"),
        normalized_weights=execution.output_file("wn.mgh"),
        signal_estimate=execution.output_file("yhat.mgh"),
        final_mask=execution.output_file("mask.mgh"),
        design_condition=execution.output_file("cond.mgh"),
        contrast_output_dir=execution.output_file("contrast1name"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_GLMFIT_METADATA",
    "MriGlmfitOutputs",
    "mri_glmfit",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SCENE_FILE_UPDATE_METADATA = Metadata(
    id="b6179bfd334c9b44420e1478f7f7ccaab55f5073.boutiques",
    name="scene-file-update",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


@dataclasses.dataclass
class SceneFileUpdateCopyMapOnePalette:
    """
    Copy palettes settings from first map to all maps in a data file.
    """
    data_file_name_suffix: str
    """Name of palette mapped data file (cifti, metric, volume)"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-copy-map-one-palette")
        cargs.append(self.data_file_name_suffix)
        return cargs


@dataclasses.dataclass
class SceneFileUpdateDataFileAdd:
    """
    Add a data file to scene's loaded files.
    """
    name_of_data_file: str
    """Name of data file. If a data file not in the current directory, it is
    best to use an absolute path (a relative path may work). Instead of a data
    file, this value may be the name of a text file (must end in ".txt") that
    contains the names of one or more data files, one per line.
    Example on UNIX to create a text file containing all CIFTI scalar files in
    the current directory with absolute paths:
    ls -d $PWD/*dscalar.nii > file.txt"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-data-file-add")
        cargs.append(self.name_of_data_file)
        return cargs


@dataclasses.dataclass
class SceneFileUpdateDataFileRemove:
    """
    Remove a data file from scene's loaded files.
    """
    name_of_data_file: str
    """Name of data file. If a data file not in the current directory, it is
    best to use an absolute path (a relative path may work). Instead of a data
    file, this value may be the name of a text file (must end in ".txt") that
    contains the names of one or more data files, one per line.
    Example on UNIX to create a text file containing all CIFTI scalar files in
    the current directory with absolute paths:
    ls -d $PWD/*dscalar.nii > file.txt"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-data-file-remove")
        cargs.append(self.name_of_data_file)
        return cargs


class SceneFileUpdateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `scene_file_update(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def scene_file_update(
    input_scene_file: str,
    output_scene_file: str,
    scene_name_or_number: str,
    opt_fix_map_palette_settings: bool = False,
    opt_remove_missing_files: bool = False,
    opt_error: bool = False,
    opt_verbose: bool = False,
    copy_map_one_palette: list[SceneFileUpdateCopyMapOnePalette] | None = None,
    data_file_add: list[SceneFileUpdateDataFileAdd] | None = None,
    data_file_remove: list[SceneFileUpdateDataFileRemove] | None = None,
    runner: Runner | None = None,
) -> SceneFileUpdateOutputs:
    """
    Update scene file.
    
    This command will update a scene for specific changes in data files.
    
    "-fix-map-palette-settings" will find all data files that have had a change
    in the number of maps since the scene scene was created. If the file has its
    "Apply to All Maps" property enabled, the palette setting in the first map
    is copied to all maps in the file. Note: This modifies the palette settings
    for the file in the scene (data file is NOT modified).
    
    "-copy-map-one-palette" will copy the palette settings from the first map to
    all other maps in a data file. This option is typically used when the number
    of maps in a data file changes. It changes the palette settings in the scene
    that are applied to the data file when the scene is loaded (the data file is
    not modified). The name of the data file specified on the command line is
    matched to the end of file names in the scene. This allows matching multiple
    files if their names end with the same characters. It also allows including
    a relative path when there is more than one file with the same name but in
    different paths and only one of the files to be updated.
    
    "-remove-missing-files" Any files that fail to load when the scene is read
    will be removed from the scene. Thus, if one deletes files prior to running
    with this option, the deleted files are removed from the scene.
    
    "-error" If this option is provided and there is an error while performing
    any of the scene operations, the command will immediately cease processing
    and the output scene file will not be created. Otherwise any errors will be
    listed after the command finishes.
    .
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        input_scene_file: the input scene file.
        output_scene_file: the new scene file to create.
        scene_name_or_number: name or number (starting at one) of the scene in\
            the scene file.
        opt_fix_map_palette_settings: Fix palette settings for files with\
            change in number of maps.
        opt_remove_missing_files: Remove missing files from SpecFile.
        opt_error: Abort command if there is an error performing any of the\
            operations on the scene file.
        opt_verbose: Print names of files that have palettes updated.
        copy_map_one_palette: Copy palettes settings from first map to all maps\
            in a data file.
        data_file_add: Add a data file to scene's loaded files.
        data_file_remove: Remove a data file from scene's loaded files.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SceneFileUpdateOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SCENE_FILE_UPDATE_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-scene-file-update")
    cargs.append(input_scene_file)
    cargs.append(output_scene_file)
    cargs.append(scene_name_or_number)
    if opt_fix_map_palette_settings:
        cargs.append("-fix-map-palette-settings")
    if opt_remove_missing_files:
        cargs.append("-remove-missing-files")
    if opt_error:
        cargs.append("-error")
    if opt_verbose:
        cargs.append("-verbose")
    if copy_map_one_palette is not None:
        cargs.extend([a for c in [s.run(execution) for s in copy_map_one_palette] for a in c])
    if data_file_add is not None:
        cargs.extend([a for c in [s.run(execution) for s in data_file_add] for a in c])
    if data_file_remove is not None:
        cargs.extend([a for c in [s.run(execution) for s in data_file_remove] for a in c])
    ret = SceneFileUpdateOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SCENE_FILE_UPDATE_METADATA",
    "SceneFileUpdateCopyMapOnePalette",
    "SceneFileUpdateDataFileAdd",
    "SceneFileUpdateDataFileRemove",
    "SceneFileUpdateOutputs",
    "scene_file_update",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

THRESHOLD_IMAGE_METADATA = Metadata(
    id="21280b87f93b4982f5b7173d35c9223a6b16962d.boutiques",
    name="ThresholdImage",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


class ThresholdImageOutputs(typing.NamedTuple):
    """
    Output object returned when calling `threshold_image(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image: OutputPathType
    """The resulting image after thresholding."""


def threshold_image(
    image_dimension: int,
    image_in: InputPathType,
    out_image: InputPathType,
    threshlo: float | None = None,
    threshhi: float | None = None,
    inside_value: float | None = None,
    outside_value: float | None = None,
    otsu_number_of_thresholds: float | None = None,
    kmeans_number_of_thresholds: float | None = None,
    mask_image: InputPathType | None = None,
    runner: Runner | None = None,
) -> ThresholdImageOutputs:
    """
    Image thresholding utility that applies different thresholding techniques to an
    input image. It can use fixed thresholds, Otsu method, or K-means for
    thresholding.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: The dimension of the input image.
        image_in: The input image file to be thresholded.
        out_image: The output image file after thresholding.
        threshlo: The lower threshold value for fixed thresholding.
        threshhi: The upper threshold value for fixed thresholding.
        inside_value: The pixel value to be used inside the threshold range.
        outside_value: The pixel value to be used outside the threshold range.
        otsu_number_of_thresholds: Number of thresholds to use when applying\
            the Otsu method.
        kmeans_number_of_thresholds: Number of thresholds to use when applying\
            the K-means method.
        mask_image: Optional mask image for the thresholding operation.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ThresholdImageOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(THRESHOLD_IMAGE_METADATA)
    cargs = []
    cargs.append("ThresholdImage")
    cargs.append(str(image_dimension))
    cargs.append(execution.input_file(image_in))
    cargs.append(execution.input_file(out_image))
    if threshlo is not None:
        cargs.append(str(threshlo))
    if threshhi is not None:
        cargs.append(str(threshhi))
    if inside_value is not None:
        cargs.append(str(inside_value))
    if outside_value is not None:
        cargs.append(str(outside_value))
    if otsu_number_of_thresholds is not None:
        cargs.append(str(otsu_number_of_thresholds))
    if kmeans_number_of_thresholds is not None:
        cargs.append(str(kmeans_number_of_thresholds))
    if mask_image is not None:
        cargs.append(execution.input_file(mask_image))
    ret = ThresholdImageOutputs(
        root=execution.output_file("."),
        output_image=execution.output_file(pathlib.Path(out_image).name),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "THRESHOLD_IMAGE_METADATA",
    "ThresholdImageOutputs",
    "threshold_image",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_RF_TRAIN_METADATA = Metadata(
    id="399c028f38b3c5c7b4be87e2dc94d14eb2c8a525.boutiques",
    name="mri_rf_train",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriRfTrainOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_rf_train(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def mri_rf_train(
    seg_volume: str,
    atlas_transform: str,
    subjects: list[str],
    output_rfa: str,
    mask_volume: str | None = None,
    node_spacing: float | None = None,
    prior_spacing: float | None = None,
    input_training_data: list[str] | None = None,
    sanity_check: bool = False,
    runner: Runner | None = None,
) -> MriRfTrainOutputs:
    """
    Trains GCA data with multiple subjects using MRI data.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        seg_volume: Segmentation volume (path relative to $subject/mri).
        atlas_transform: Atlas transform (path relative to\
            $subject/mri/transforms).
        subjects: Input subjects.
        output_rfa: Output RFA filename.
        mask_volume: Use volname as a mask (path relative to $subject/mri).
        node_spacing: Spacing of classifiers in canonical space.
        prior_spacing: Spacing of class priors in canonical space.
        input_training_data: Specifying training data (path relative to\
            $subject/mri). Can specify multiple inputs. If not specified, 'orig' is\
            used.
        sanity_check: Conduct sanity-check of labels for obvious edit errors.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriRfTrainOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_RF_TRAIN_METADATA)
    cargs = []
    cargs.append("mri_rf_train")
    cargs.extend([
        "-seg",
        seg_volume
    ])
    cargs.extend([
        "-xform",
        atlas_transform
    ])
    if mask_volume is not None:
        cargs.extend([
            "-mask",
            mask_volume
        ])
    if node_spacing is not None:
        cargs.extend([
            "-node_spacing",
            str(node_spacing)
        ])
    if prior_spacing is not None:
        cargs.extend([
            "-prior_spacing",
            str(prior_spacing)
        ])
    if input_training_data is not None:
        cargs.extend([
            "-input",
            *input_training_data
        ])
    if sanity_check:
        cargs.append("-check")
    cargs.extend(subjects)
    cargs.append(output_rfa)
    ret = MriRfTrainOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_RF_TRAIN_METADATA",
    "MriRfTrainOutputs",
    "mri_rf_train",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

APQC_MAKE_TCSH_PY_METADATA = Metadata(
    id="9963574d56b682af5542493c6e37b26eeae1361e.boutiques",
    name="apqc_make_tcsh.py",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class ApqcMakeTcshPyOutputs(typing.NamedTuple):
    """
    Output object returned when calling `apqc_make_tcsh_py(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    qc_html: OutputPathType
    """Quality Control HTML file generated by APQC."""


def apqc_make_tcsh_py(
    uvar_json: InputPathType,
    subj_dir: str,
    review_style: str | None = None,
    mot_grayplot_off: bool = False,
    vstat_list: list[str] | None = None,
    runner: Runner | None = None,
) -> ApqcMakeTcshPyOutputs:
    """
    This program creates the single subject (ss) HTML review script
    '@ss_review_html' which generates images and text for the afni_proc.py quality
    control (APQC) HTML.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        uvar_json: Text file of user variables created by gen_ss_review.py that\
            catalogs important files in the results directory for the APQC.
        subj_dir: Location of AP results directory (often '.', as this program\
            is often run from within the AP results directory).
        review_style: The 'style' of the APQC HTML output HTML. Allowed\
            keywords are: {none, basic, pythonic}. Using 'pythonic' is recommended.
        mot_grayplot_off: Turn off the grayplot generation. This option was\
            created for a specific case with a large dataset. Not recommended to\
            use generally.
        vstat_list: Provide a list of label items to specify which volume's\
            images should appear in the vstat QC block. Each item should correspond\
            to subbrick label basename in the stats_dset. 'Full_Fstat' is always\
            included. If not used, default logic picks up to 5 items to show.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ApqcMakeTcshPyOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(APQC_MAKE_TCSH_PY_METADATA)
    cargs = []
    cargs.append("apqc_make_tcsh.py")
    cargs.extend([
        "-uvar_json",
        execution.input_file(uvar_json)
    ])
    cargs.extend([
        "-subj_dir",
        subj_dir
    ])
    if review_style is not None:
        cargs.extend([
            "-review_style",
            review_style
        ])
    if mot_grayplot_off:
        cargs.append("-mot_grayplot_off")
    if vstat_list is not None:
        cargs.extend([
            "-vstat_list",
            *vstat_list
        ])
    ret = ApqcMakeTcshPyOutputs(
        root=execution.output_file("."),
        qc_html=execution.output_file("qc_*"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "APQC_MAKE_TCSH_PY_METADATA",
    "ApqcMakeTcshPyOutputs",
    "apqc_make_tcsh_py",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

ANTS_DENOISE_IMAGE_FS_METADATA = Metadata(
    id="11723bf924918f42194dab2a080f4af8786d3033.boutiques",
    name="AntsDenoiseImageFs",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class AntsDenoiseImageFsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ants_denoise_image_fs(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    denoised_output: OutputPathType
    """Resulting denoised volume file"""


def ants_denoise_image_fs(
    input_image: InputPathType,
    output_image: str = "output.nii",
    rician_flag: bool = False,
    runner: Runner | None = None,
) -> AntsDenoiseImageFsOutputs:
    """
    Denoises an image with a spatially adaptive filter. This program wraps the
    AntsDenoiseImage utility available in the ANTs package.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_image: Input volume file.
        output_image: Denoised volume file.
        rician_flag: Enable Rician noise model (otherwise Gaussian is used).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsDenoiseImageFsOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_DENOISE_IMAGE_FS_METADATA)
    cargs = []
    cargs.append("AntsDenoiseImageFs")
    cargs.extend([
        "-i",
        execution.input_file(input_image)
    ])
    cargs.extend([
        "-o",
        output_image
    ])
    if rician_flag:
        cargs.append("--rician")
    ret = AntsDenoiseImageFsOutputs(
        root=execution.output_file("."),
        denoised_output=execution.output_file(output_image),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "ANTS_DENOISE_IMAGE_FS_METADATA",
    "AntsDenoiseImageFsOutputs",
    "ants_denoise_image_fs",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

NICCC_METADATA = Metadata(
    id="ecc865fc2143792b2c5c1c0adc6ffbd967c02c7f.boutiques",
    name="niccc",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class NicccOutputs(typing.NamedTuple):
    """
    Output object returned when calling `niccc(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    stderr_output: OutputPathType
    """Results output to stderr"""


def niccc(
    streamspec: str,
    duplicate: bool = False,
    nodata: bool = False,
    attribute: str | None = None,
    match: str | None = None,
    file: bool = False,
    string_: bool = False,
    stdout: bool = False,
    hash_: bool = False,
    quiet: bool = False,
    find_attr: list[str] | None = None,
    skip_attr: list[str] | None = None,
    runner: Runner | None = None,
) -> NicccOutputs:
    """
    A program for conducting certain NIML tests on input from streamspec and write
    the results to stderr.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        streamspec: A string defining a NIML stream.
        duplicate: Duplicate the element before showing it. This is to test\
            NI_duplicate function.
        nodata: Show header parts only in output.
        attribute: Dump the value of attribute ATTR.
        match: Match attribute: If MATCH is exact, then attribute name is\
            matched exactly. If MATCH is partial, then a match of all the\
            characters in ATTR is enough.
        file: Streamspec is a filename.
        string_: Streamspec is an element string like: '<T font=9 coords="2.3\
            23 2"/>'.
        stdout: Write elements to stdout, instead of stderr.
        hash_: Put the # at the beginning of lines with no data.
        quiet: Quiet stderr messages, and don't echo attribute name with\
            -attribute option.
        find_attr: Only output elements that have an attribute ATTR of value\
            ATTRVAL.
        skip_attr: Do not output elements that have an attribute ATTR of value\
            ATTRVAL.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `NicccOutputs`).
    """
    if find_attr is not None and (len(find_attr) != 2): 
        raise ValueError(f"Length of 'find_attr' must be 2 but was {len(find_attr)}")
    if skip_attr is not None and (len(skip_attr) != 2): 
        raise ValueError(f"Length of 'skip_attr' must be 2 but was {len(skip_attr)}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(NICCC_METADATA)
    cargs = []
    cargs.append("niccc")
    cargs.append(streamspec)
    if duplicate:
        cargs.append("-dup")
    if nodata:
        cargs.append("-nodata")
    if attribute is not None:
        cargs.extend([
            "-attribute",
            attribute
        ])
    if match is not None:
        cargs.extend([
            "-match",
            match
        ])
    if file:
        cargs.append("-f")
    if string_:
        cargs.append("-s")
    if stdout:
        cargs.append("-stdout")
    if hash_:
        cargs.append("-#")
    if quiet:
        cargs.append("-quiet")
    if find_attr is not None:
        cargs.extend([
            "-find_nel_with_attr",
            *find_attr
        ])
    if skip_attr is not None:
        cargs.extend([
            "-skip_nel_with_attr",
            *skip_attr
        ])
    ret = NicccOutputs(
        root=execution.output_file("."),
        stderr_output=execution.output_file("stderr"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "NICCC_METADATA",
    "NicccOutputs",
    "niccc",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

CONVERTWARP_METADATA = Metadata(
    id="cd7b2e6bda3ae320e54128fba7348ed3eb91d604.boutiques",
    name="convertwarp",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


class ConvertwarpOutputs(typing.NamedTuple):
    """
    Output object returned when calling `convertwarp(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_file: OutputPathType
    """Name of output file, containing warps that are the combination of all
    those given as arguments. the format of this will be a field-file (rather
    than spline coefficients) with any affine components included."""
    out_file_: OutputPathType
    """Name of output file, containing the warp as field or coefficients."""


def convertwarp(
    reference: InputPathType,
    abswarp: bool = False,
    cons_jacobian: bool = False,
    jacobian_max: float | None = None,
    jacobian_min: float | None = None,
    midmat: InputPathType | None = None,
    out_abswarp: bool = False,
    out_relwarp: bool = False,
    output_type: typing.Literal["NIFTI", "NIFTI_PAIR", "NIFTI_GZ", "NIFTI_PAIR_GZ"] | None = None,
    postmat: InputPathType | None = None,
    premat: InputPathType | None = None,
    relwarp: bool = False,
    shift_direction: typing.Literal["y-", "y", "x", "x-", "z", "z-"] | None = None,
    shift_in_file: InputPathType | None = None,
    warp1: InputPathType | None = None,
    warp2: InputPathType | None = None,
    runner: Runner | None = None,
) -> ConvertwarpOutputs:
    """
    Use FSL convertwarp for combining multiple transforms into one.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        reference: Name of a file in target space of the full transform.
        abswarp: If set it indicates that the warps in --warp1 and --warp2\
            should be interpreted as absolute. i.e. the values in --warp1/2 are the\
            coordinates in the next space, rather than displacements. this flag is\
            ignored if --warp1/2 was created by fnirt, which always creates\
            relative displacements.
        cons_jacobian: Constrain the jacobian of the warpfield to lie within\
            specified min/max limits.
        jacobian_max: Maximum acceptable jacobian value for constraint (default\
            100.0).
        jacobian_min: Minimum acceptable jacobian value for constraint (default\
            0.01).
        midmat: Name of file containing mid-warp-affine transform.
        out_abswarp: If set it indicates that the warps in --out should be\
            absolute, i.e. the values in --out are displacements from the\
            coordinates in --ref.
        out_relwarp: If set it indicates that the warps in --out should be\
            relative, i.e. the values in --out are displacements from the\
            coordinates in --ref.
        output_type: 'nifti' or 'nifti_pair' or 'nifti_gz' or 'nifti_pair_gz'.\
            Fsl output type.
        postmat: Name of file containing an affine transform (applied last). it\
            could e.g. be an affine transform that maps the mni152-space into a\
            better approximation to the talairach-space (if indeed there is one).
        premat: Filename for pre-transform (affine matrix).
        relwarp: If set it indicates that the warps in --warp1/2 should be\
            interpreted as relative. i.e. the values in --warp1/2 are displacements\
            from the coordinates in the next space.
        shift_direction: 'y-' or 'y' or 'x' or 'x-' or 'z' or 'z-'. Indicates\
            the direction that the distortions from --shiftmap goes. it depends on\
            the direction and polarity of the phase-encoding in the epi sequence.
        shift_in_file: Name of file containing a "shiftmap", a non-linear\
            transform with displacements only in one direction (applied first,\
            before premat). this would typically be a fieldmap that has been\
            pre-processed using fugue that maps a subjects functional (epi) data\
            onto an undistorted space (i.e. a space that corresponds to his/her\
            true anatomy).
        warp1: Name of file containing initial warp-fields/coefficients\
            (follows premat). this could e.g. be a fnirt-transform from a subjects\
            structural scan to an average of a group of subjects.
        warp2: Name of file containing secondary warp-fields/coefficients\
            (after warp1/midmat but before postmat). this could e.g. be a\
            fnirt-transform from the average of a group of subjects to some\
            standard space (e.g. mni152).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConvertwarpOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CONVERTWARP_METADATA)
    cargs = []
    cargs.append("convertwarp")
    if abswarp:
        cargs.append("--abs")
    if cons_jacobian:
        cargs.append("--constrainj")
    if jacobian_max is not None:
        cargs.append("--jmax=" + str(jacobian_max))
    if jacobian_min is not None:
        cargs.append("--jmin=" + str(jacobian_min))
    if midmat is not None:
        cargs.append("--midmat=" + execution.input_file(midmat))
    if out_abswarp:
        cargs.append("--absout")
    if out_relwarp:
        cargs.append("--relout")
    if output_type is not None:
        cargs.append(output_type)
    if postmat is not None:
        cargs.append("--postmat=" + execution.input_file(postmat))
    if premat is not None:
        cargs.append("--premat=" + execution.input_file(premat))
    cargs.append("--ref=" + execution.input_file(reference))
    if relwarp:
        cargs.append("--rel")
    if shift_direction is not None:
        cargs.append("--shiftdir=" + shift_direction)
    if shift_in_file is not None:
        cargs.append("--shiftmap=" + execution.input_file(shift_in_file))
    if warp1 is not None:
        cargs.append("--warp1=" + execution.input_file(warp1))
    if warp2 is not None:
        cargs.append("--warp2=" + execution.input_file(warp2))
    ret = ConvertwarpOutputs(
        root=execution.output_file("."),
        out_file=execution.output_file(pathlib.Path(reference).name + "_concatwarp"),
        out_file_=execution.output_file("out_file"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "CONVERTWARP_METADATA",
    "ConvertwarpOutputs",
    "convertwarp",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MAKE_SYMMETRIC_METADATA = Metadata(
    id="0edab2e26b68afca9fa6531a0a4823dd6acee05b.boutiques",
    name="make_symmetric",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MakeSymmetricOutputs(typing.NamedTuple):
    """
    Output object returned when calling `make_symmetric(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    processed_output: OutputPathType
    """Output image file after processing."""
    map_output: OutputPathType
    """Transformation map file mapping the input to the upright space."""


def make_symmetric(
    hemi: str,
    input_file: InputPathType,
    output_file: str,
    transform_map: str,
    runner: Runner | None = None,
) -> MakeSymmetricOutputs:
    """
    Registers an input image to its left/right reversed version using
    mri_robust_register in a half-way space and mirrors the selected hemisphere.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        hemi: The hemisphere to mirror; accepted values are 'lh' for left\
            hemisphere or 'rh' for right hemisphere.
        input_file: Input image in .mgz format.
        output_file: Output image in .mgz format resulting from the symmetric\
            processing.
        transform_map: Transformation map in .lta format mapping the input to\
            the upright space.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MakeSymmetricOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MAKE_SYMMETRIC_METADATA)
    cargs = []
    cargs.append("make_symmetric")
    cargs.append(hemi)
    cargs.append(execution.input_file(input_file))
    cargs.append(output_file)
    cargs.append(transform_map)
    ret = MakeSymmetricOutputs(
        root=execution.output_file("."),
        processed_output=execution.output_file(output_file),
        map_output=execution.output_file(transform_map),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MAKE_SYMMETRIC_METADATA",
    "MakeSymmetricOutputs",
    "make_symmetric",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

IMUPSAM_METADATA = Metadata(
    id="156965091ca5242419f26a19463380332f6a9731.boutiques",
    name="imupsam",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class ImupsamOutputs(typing.NamedTuple):
    """
    Output object returned when calling `imupsam(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image_file: OutputPathType
    """Upsampled image output file"""


def imupsam(
    factor: int,
    input_image: InputPathType,
    output_image: str,
    ascii_flag: bool = False,
    runner: Runner | None = None,
) -> ImupsamOutputs:
    """
    Upsamples a 2D image by a specified factor.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        factor: Upsampling factor; must be an integer in the range 2 to 30.
        input_image: Path of the input 2D image file.
        output_image: Path of the output upsampled image file. Use '-' to write\
            to stdout.
        ascii_flag: Write the result in ASCII format: all numbers for the file\
            are output, with no header info.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ImupsamOutputs`).
    """
    if not (2 <= factor <= 30): 
        raise ValueError(f"'factor' must be between 2 <= x <= 30 but was {factor}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(IMUPSAM_METADATA)
    cargs = []
    cargs.append("imupsam")
    if ascii_flag:
        cargs.append("-A")
    cargs.append(str(factor))
    cargs.append(execution.input_file(input_image))
    cargs.append(output_image)
    ret = ImupsamOutputs(
        root=execution.output_file("."),
        output_image_file=execution.output_file(output_image),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "IMUPSAM_METADATA",
    "ImupsamOutputs",
    "imupsam",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

HIAM_MAKE_TEMPLATE_METADATA = Metadata(
    id="8c48bbfd46c7f8da62fb53ac9c94dd28311f4d1a.boutiques",
    name="hiam_make_template",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class HiamMakeTemplateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `hiam_make_template(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def hiam_make_template(
    hemi: str,
    surface_name: str,
    subjects: list[str],
    output_name: str,
    runner: Runner | None = None,
) -> HiamMakeTemplateOutputs:
    """
    This program adds a template into an average surface using FreeSurfer.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        hemi: Hemisphere to be processed (e.g. lh or rh).
        surface_name: Name of the surface.
        subjects: List of subject identifiers.
        output_name: Name of the output template.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `HiamMakeTemplateOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(HIAM_MAKE_TEMPLATE_METADATA)
    cargs = []
    cargs.append("hiam_make_template")
    cargs.append(hemi)
    cargs.append(surface_name)
    cargs.extend(subjects)
    cargs.append(output_name)
    ret = HiamMakeTemplateOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "HIAM_MAKE_TEMPLATE_METADATA",
    "HiamMakeTemplateOutputs",
    "hiam_make_template",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_HAUSDORFF_DIST_METADATA = Metadata(
    id="c8c6f8e737d084817096cd5c6491a8218fa2f1ee.boutiques",
    name="mris_hausdorff_dist",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisHausdorffDistOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_hausdorff_dist(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType
    """Output file containing the Hausdorff distance computation results"""


def mris_hausdorff_dist(
    surface: InputPathType,
    label1: InputPathType,
    label2: InputPathType,
    annot_name: str | None = None,
    runner: Runner | None = None,
) -> MrisHausdorffDistOutputs:
    """
    This program computes the Hausdorff distance between two labels on a surface.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        surface: Surface file on which the labels exist.
        label1: First label file.
        label2: Second label file.
        annot_name: Compute pairwise Hausdorff Distance (HD) between all\
            annotations with the given name.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisHausdorffDistOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_HAUSDORFF_DIST_METADATA)
    cargs = []
    cargs.append("mris_hausdorff_dist")
    cargs.append(execution.input_file(surface))
    cargs.append(execution.input_file(label1))
    cargs.append(execution.input_file(label2))
    if annot_name is not None:
        cargs.extend([
            "-a",
            annot_name
        ])
    ret = MrisHausdorffDistOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file("hausdorff_output.txt"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_HAUSDORFF_DIST_METADATA",
    "MrisHausdorffDistOutputs",
    "mris_hausdorff_dist",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_EDIT_WM_WITH_ASEG_METADATA = Metadata(
    id="f9c976c7dcbdda3256a48669b4d7c0911a56357c.boutiques",
    name="mri_edit_wm_with_aseg",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriEditWmWithAsegOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_edit_wm_with_aseg(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_wm_file: OutputPathType
    """Edited white matter output file"""


def mri_edit_wm_with_aseg(
    input_wm: InputPathType,
    input_t1_brain: InputPathType,
    aseg: InputPathType,
    output_wm: str,
    fillven: bool = False,
    fix_scm_ha: int | None = None,
    fix_scm_ha_only: str | None = None,
    keep: bool = False,
    keep_in: bool = False,
    lh: bool = False,
    rh: bool = False,
    fix_ento_wm: str | None = None,
    sa_fix_ento_wm: str | None = None,
    debug_voxel: list[float] | None = None,
    runner: Runner | None = None,
) -> MriEditWmWithAsegOutputs:
    """
    A tool for editing white matter with anatomical segmentation.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_wm: Input white matter file.
        input_t1_brain: Input T1/brain file.
        aseg: Anatomical segmentation file.
        output_wm: Output white matter file.
        fillven: Fill ventricular system.
        fix_scm_ha: Remove voxels in amygdala, ILV, and parts of hippocampus.
        fix_scm_ha_only: Standalone: fix SCM using aseg.presurf.mgz.
        keep: Keep edits as found in output volume.
        keep_in: Keep edits as found in input volume.
        lh: Erase right hemisphere labels from output.
        rh: Erase left hemisphere labels from output.
        fix_ento_wm: Insert lhval rhval where {3,4}006 and {3,4}201 in entowm\
            volume.
        sa_fix_ento_wm: Standalone version of fix ento-WM.
        debug_voxel: Specify a voxel to edit with coordinates Gx Gy Gz.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriEditWmWithAsegOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_EDIT_WM_WITH_ASEG_METADATA)
    cargs = []
    cargs.append("mri_edit_wm_with_aseg")
    cargs.append(execution.input_file(input_wm))
    cargs.append(execution.input_file(input_t1_brain))
    cargs.append(execution.input_file(aseg))
    cargs.append(output_wm)
    if fillven:
        cargs.append("-fillven")
    if fix_scm_ha is not None:
        cargs.extend([
            "-fix-scm-ha",
            str(fix_scm_ha)
        ])
    if fix_scm_ha_only is not None:
        cargs.extend([
            "-fix-scm-ha-only",
            fix_scm_ha_only
        ])
    if keep:
        cargs.append("-keep")
    if keep_in:
        cargs.append("-keep-in")
    if lh:
        cargs.append("-lh")
    if rh:
        cargs.append("-rh")
    if fix_ento_wm is not None:
        cargs.extend([
            "-fix-ento-wm",
            fix_ento_wm
        ])
    if sa_fix_ento_wm is not None:
        cargs.extend([
            "-sa-fix-ento-wm",
            sa_fix_ento_wm
        ])
    if debug_voxel is not None:
        cargs.extend([
            "-debug_voxel",
            *map(str, debug_voxel)
        ])
    ret = MriEditWmWithAsegOutputs(
        root=execution.output_file("."),
        output_wm_file=execution.output_file(output_wm),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_EDIT_WM_WITH_ASEG_METADATA",
    "MriEditWmWithAsegOutputs",
    "mri_edit_wm_with_aseg",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SURFACE_SPHERE_TRIANGULAR_PATCHES_METADATA = Metadata(
    id="0041e957f3f86307d53f30986bba8e52c79c3ab8.boutiques",
    name="surface-sphere-triangular-patches",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


class SurfaceSphereTriangularPatchesOutputs(typing.NamedTuple):
    """
    Output object returned when calling `surface_sphere_triangular_patches(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def surface_sphere_triangular_patches(
    sphere: InputPathType,
    divisions: int,
    text_out: str,
    runner: Runner | None = None,
) -> SurfaceSphereTriangularPatchesOutputs:
    """
    Divide standard sphere into patches.
    
    Divide the given undistorted sphere into equally-sized triangular patches.
    Patches overlap by a border of 1 vertex.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        sphere: an undistorted, regularly divided icosahedral sphere.
        divisions: how many pieces to divide each icosahedral edge into, must\
            divide perfectly into the given sphere.
        text_out: output - text file for the vertex numbers of the patches.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceSphereTriangularPatchesOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_SPHERE_TRIANGULAR_PATCHES_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-surface-sphere-triangular-patches")
    cargs.append(execution.input_file(sphere))
    cargs.append(str(divisions))
    cargs.append(text_out)
    ret = SurfaceSphereTriangularPatchesOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SURFACE_SPHERE_TRIANGULAR_PATCHES_METADATA",
    "SurfaceSphereTriangularPatchesOutputs",
    "surface_sphere_triangular_patches",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_PARSE_SDCMDIR_METADATA = Metadata(
    id="7b3e6193afc93ee99689a8a78865b9a043556c85.boutiques",
    name="mri_parse_sdcmdir",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriParseSdcmdirOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_parse_sdcmdir(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def mri_parse_sdcmdir(
    sdicomdir: str,
    outfile: str | None = None,
    sortbyrun: bool = False,
    summarize: bool = False,
    dwi: bool = False,
    runner: Runner | None = None,
) -> MriParseSdcmdirOutputs:
    """
    This program parses the Siemens DICOM files in a given directory and prints out
    information about each file.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        sdicomdir: Path to Siemens DICOM directory.
        outfile: Name of a file to which the results will be printed. If\
            unspecified, the results will be printed to stdout.
        sortbyrun: Assign run numbers.
        summarize: Only print out info for run leaders.
        dwi: Try to read DWI params. Generally no need to.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriParseSdcmdirOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_PARSE_SDCMDIR_METADATA)
    cargs = []
    cargs.append("mri_parse_sdcmdir")
    cargs.extend([
        "--d",
        sdicomdir
    ])
    if outfile is not None:
        cargs.extend([
            "--o",
            outfile
        ])
    if sortbyrun:
        cargs.append("--sortbyrun")
    if summarize:
        cargs.append("--summarize")
    if dwi:
        cargs.append("--dwi")
    ret = MriParseSdcmdirOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_PARSE_SDCMDIR_METADATA",
    "MriParseSdcmdirOutputs",
    "mri_parse_sdcmdir",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

R_PKGS_INSTALL_METADATA = Metadata(
    id="9c9d4c7ec7df54a4bd835bc8c7156ea51895da44.boutiques",
    name="rPkgsInstall",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class RPkgsInstallOutputs(typing.NamedTuple):
    """
    Output object returned when calling `r_pkgs_install(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_packages: OutputPathType
    """Output R packages after installation, update, or removal"""


def r_pkgs_install(
    packages: str,
    download_site: str | None = None,
    check: bool = False,
    update_: bool = False,
    remove: bool = False,
    runner: Runner | None = None,
) -> RPkgsInstallOutputs:
    """
    A tool for installing, checking, updating, or removing R packages for AFNI.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        packages: List of R packages to install, update, or remove. Use 'ALL'\
            to refer to all AFNI-required packages.
        download_site: Specify the package repository website. Default is\
            'http://cloud.r-project.org'.
        check: Verify whether the specified R packages are installed on the\
            computer without installing/updating/removing them.
        update_: Update the specified R packages. If packages are not\
            installed, they will be installed.
        remove: Remove the specified R packages from the system.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `RPkgsInstallOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(R_PKGS_INSTALL_METADATA)
    cargs = []
    cargs.append("rPkgsInstall")
    cargs.extend([
        "-pkgs",
        packages
    ])
    if download_site is not None:
        cargs.extend([
            "-site",
            download_site
        ])
    if check:
        cargs.append("-check")
    if update_:
        cargs.append("-update")
    if remove:
        cargs.append("-remove")
    ret = RPkgsInstallOutputs(
        root=execution.output_file("."),
        output_packages=execution.output_file(packages),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "RPkgsInstallOutputs",
    "R_PKGS_INSTALL_METADATA",
    "r_pkgs_install",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_GRADIENT_METADATA = Metadata(
    id="4e76ee59424b712977309da37bac35a8195a7230.boutiques",
    name="mris_gradient",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisGradientOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_gradient(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_gradient: OutputPathType
    """The resulting gradient measurement written into a .mgz file."""


def mris_gradient(
    input_surface: InputPathType,
    input_vector_field: InputPathType,
    output_gradient_file: str,
    runner: Runner | None = None,
) -> MrisGradientOutputs:
    """
    This program computes the gradient of an intensity profile of the cortical
    ribbon and writes the resulting measurement into a .mgz file.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_surface: Path to the input surface file.
        input_vector_field: Path to the input vector field file.
        output_gradient_file: Path to the output gradient file ending with\
            .mgz.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisGradientOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_GRADIENT_METADATA)
    cargs = []
    cargs.append("mris_gradient")
    cargs.append(execution.input_file(input_surface))
    cargs.append(execution.input_file(input_vector_field))
    cargs.append(output_gradient_file)
    ret = MrisGradientOutputs(
        root=execution.output_file("."),
        output_gradient=execution.output_file(output_gradient_file),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_GRADIENT_METADATA",
    "MrisGradientOutputs",
    "mris_gradient",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_ANNOT_TO_SEGMENTATION_METADATA = Metadata(
    id="a6fe77811c128028277a69a99f1e7f945cb0ac57.boutiques",
    name="mris_annot_to_segmentation",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisAnnotToSegmentationOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_annot_to_segmentation(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_volume_file: OutputPathType
    """Generated segmentation volume."""


def mris_annot_to_segmentation(
    subject_name: str,
    hemi: str,
    surface: str,
    annot_file: InputPathType,
    color_table: InputPathType,
    output_volume: str,
    runner: Runner | None = None,
) -> MrisAnnotToSegmentationOutputs:
    """
    Converts annotation files to segmentation volumes in FreeSurfer.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subject_name: Subject name.
        hemi: Hemisphere (e.g., lh or rh).
        surface: Surface file.
        annot_file: Annotation file.
        color_table: Color table file.
        output_volume: Output volume file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisAnnotToSegmentationOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_ANNOT_TO_SEGMENTATION_METADATA)
    cargs = []
    cargs.append("mris_annot_to_segmentation")
    cargs.append(subject_name)
    cargs.append(hemi)
    cargs.append(surface)
    cargs.append(execution.input_file(annot_file))
    cargs.append(execution.input_file(color_table))
    cargs.append(output_volume)
    ret = MrisAnnotToSegmentationOutputs(
        root=execution.output_file("."),
        output_volume_file=execution.output_file(output_volume),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_ANNOT_TO_SEGMENTATION_METADATA",
    "MrisAnnotToSegmentationOutputs",
    "mris_annot_to_segmentation",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_PLACE_SURFACE_METADATA = Metadata(
    id="e4e973bfe64ca5b7e38915757ccc306ea96b6327.boutiques",
    name="mris_place_surface",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisPlaceSurfaceOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_place_surface(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def mris_place_surface(
    output_surface: str,
    input_surface: str,
    autodetect_gray_white_stats: str,
    input_volume: str,
    surface_type_group: typing.Literal["--white", "--pial"],
    hemi_group: typing.Literal["--lh", "--rh"],
    wm_segment: str | None = None,
    out_volume: str | None = None,
    out_volume_only: str | None = None,
    restore_255: bool = False,
    segmentation: str | None = None,
    cortical_parcellation: str | None = None,
    nsmooth: float | None = None,
    smooth_after_rip: bool = False,
    max_cbv_dist: float | None = None,
    rip_label: str | None = None,
    rip_midline: bool = False,
    rip_bg: bool = False,
    rip_bg_no_annot: bool = False,
    no_rip_freeze: bool = False,
    rip_wmsa: bool = False,
    rip_lesion: bool = False,
    no_rip: bool = False,
    rip_overlay: str | None = None,
    rip_surface: str | None = None,
    rip_projection: list[float] | None = None,
    repulse_surface: str | None = None,
    white_surface: str | None = None,
    blend_surface: str | None = None,
    multimodal_input: str | None = None,
    mm_refine: bool = False,
    pin_medial_wall: str | None = None,
    no_intensity_proc: bool = False,
    debug_vertex: float | None = None,
    ripflag_out: str | None = None,
    local_max: str | None = None,
    target_surf: str | None = None,
    stop_mask: bool = False,
    mm_intensity_limits: str | None = None,
    cover_seg: str | None = None,
    first_peak_d1: bool = False,
    first_peak_d2: bool = False,
    white_border_low_factor: float | None = None,
    fill_lateral_ventricles: list[float] | None = None,
    runner: Runner | None = None,
) -> MrisPlaceSurfaceOutputs:
    """
    This program positions the triangular mesh representing a cortical surface,
    either the 'white' surface (ie, white/gray boundary) or the 'pial' surface (ie,
    the gray/csf boundary).
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        output_surface: Output surface.
        input_surface: Input surface.
        autodetect_gray_white_stats: Intensity stats created by\
            mris_autodet_gwstats.
        input_volume: T1-weighed intensity volume used to find white/gray/csf\
            gradients (usually brain.finalsurf.mgz).
        surface_type_group: Place the white surface or the pial surface. Must\
            choose one.
        hemi_group: Left or right hemisphere. Must choose one.
        wm_segment: White matter segmentation.
        out_volume: Save input volume after preprocessing.
        out_volume_only: Save input volume after preprocessing and then exit.
        restore_255: Set voxels in the input volume that start off as 255 to\
            110 (white surf only).
        segmentation: Whole-brain segmentation (usually aseg.presurf.mgz).
        cortical_parcellation: Set cortical parcellation used to rip vertices\
            (usually ?h.aparc.annot).
        nsmooth: Smooth input surface by number of iterations.
        smooth_after_rip: Smooth after ripping.
        max_cbv_dist: Limit distance MRIScomputeBorderValues() can search from\
            the input.
        rip_label: Do not move vertices that are NOT in the cortex label.
        rip_midline: Do not move vertices that are in the midline as indicated\
            by the seg.
        rip_bg: Do not move vertices near basal ganglia (as defined by seg).
        rip_bg_no_annot: Do not require surface have an annotation when ripping\
            BG.
        no_rip_freeze: Do NOT move vertices in/near freeze voxels (247 as\
            defined in seg).
        rip_wmsa: Do not move vertices in/near white-matter signal\
            abnormalities (77,78,79 as defined in seg).
        rip_lesion: Do not move vertices in/near lesions (25 and 57 as defined\
            in seg).
        no_rip: Turn off all ripping.
        rip_overlay: Rip vertices > 0.5 in the surface overlay file.
        rip_surface: Use this surface with ripping midline, BG, Freezes,\
            Lesions, and WMSA.
        rip_projection: Control projection depth along normal to ripsurface\
            when sampling seg.
        repulse_surface: Force input surface away from this surface (usually\
            the white surface when placing the pial).
        white_surface: Set the white{xyz} coordinates of the input surface\
            using this surface.
        blend_surface: Recompute the xyz coordinates of the input surface by\
            computing a weighted average with the blend surface.
        multimodal_input: Specify a T2 or FLAIR input volume used for placing\
            the pial surface. Must be in registration with the input volume.
        mm_refine: Use Siless' MultimodalRefinement. Sets tspring=nspring=0.3.
        pin_medial_wall: Set coordinates in vertices NOT in cortexlabel to be\
            that of the white{xyz} coordinates.
        no_intensity_proc: Do not process the input intensity volume (eg, to\
            remove parts of eye socket).
        debug_vertex: Debug vertex number.
        ripflag_out: Save ripflag as overlay.
        local_max: Save LocalMaxFoundFlag as overlay.
        target_surf: Save CBV target surface.
        stop_mask: Stop mask to stop search along normal for max gradient.
        mm_intensity_limits: Intensity limits for placing pial on multimodal\
            input.
        cover_seg: Force surface to cover the segmentation.
        first_peak_d1: Use find-first-peak option with 1st derivative in\
            ComputeBorderValues.
        first_peak_d2: Use find-first-peak option with 2nd derivative in\
            ComputeBorderValues.
        white_border_low_factor: white_border_low = f*adgws.gray_mean +\
            (1-f)*adgws.white_mean;.
        fill_lateral_ventricles: Fill lateral ventricles with 110.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisPlaceSurfaceOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_PLACE_SURFACE_METADATA)
    cargs = []
    cargs.append("mris_place_surface")
    cargs.extend([
        "--o",
        output_surface
    ])
    cargs.extend([
        "--i",
        input_surface
    ])
    cargs.extend([
        "--adgw",
        autodetect_gray_white_stats
    ])
    cargs.extend([
        "--invol",
        input_volume
    ])
    cargs.append(surface_type_group)
    cargs.append(hemi_group)
    if wm_segment is not None:
        cargs.extend([
            "--wm",
            wm_segment
        ])
    if out_volume is not None:
        cargs.extend([
            "--outvol",
            out_volume
        ])
    if out_volume_only is not None:
        cargs.extend([
            "--outvol-only",
            out_volume_only
        ])
    if restore_255:
        cargs.append("--restore-255")
    if segmentation is not None:
        cargs.extend([
            "--seg",
            segmentation
        ])
    if cortical_parcellation is not None:
        cargs.extend([
            "--aparc",
            cortical_parcellation
        ])
    if nsmooth is not None:
        cargs.extend([
            "--nsmooth",
            str(nsmooth)
        ])
    if smooth_after_rip:
        cargs.append("--smooth-after-rip")
    if max_cbv_dist is not None:
        cargs.extend([
            "--max-cbv-dist",
            str(max_cbv_dist)
        ])
    if rip_label is not None:
        cargs.extend([
            "--rip-label",
            rip_label
        ])
    if rip_midline:
        cargs.append("--rip-midline")
    if rip_bg:
        cargs.append("--rip-bg")
    if rip_bg_no_annot:
        cargs.append("--rip-bg-no-annot")
    if no_rip_freeze:
        cargs.append("--no-rip-freeze")
    if rip_wmsa:
        cargs.append("--rip-wmsa")
    if rip_lesion:
        cargs.append("--rip-lesion")
    if no_rip:
        cargs.append("--no-rip")
    if rip_overlay is not None:
        cargs.extend([
            "--rip-overlay",
            rip_overlay
        ])
    if rip_surface is not None:
        cargs.extend([
            "--ripsurface",
            rip_surface
        ])
    if rip_projection is not None:
        cargs.extend([
            "--rip-projection",
            *map(str, rip_projection)
        ])
    if repulse_surface is not None:
        cargs.extend([
            "--repulse-surf",
            repulse_surface
        ])
    if white_surface is not None:
        cargs.extend([
            "--white-surf",
            white_surface
        ])
    if blend_surface is not None:
        cargs.extend([
            "--blend-surf",
            blend_surface
        ])
    if multimodal_input is not None:
        cargs.extend([
            "--mmvol",
            multimodal_input
        ])
    if mm_refine:
        cargs.append("--mm-refine")
    if pin_medial_wall is not None:
        cargs.extend([
            "--pin-medial-wall",
            pin_medial_wall
        ])
    if no_intensity_proc:
        cargs.append("--no-intensity-proc")
    if debug_vertex is not None:
        cargs.extend([
            "--debug-vertex",
            str(debug_vertex)
        ])
    if ripflag_out is not None:
        cargs.extend([
            "--ripflag-out",
            ripflag_out
        ])
    if local_max is not None:
        cargs.extend([
            "--local-max",
            local_max
        ])
    if target_surf is not None:
        cargs.extend([
            "--target",
            target_surf
        ])
    if stop_mask:
        cargs.append("--stop")
    if mm_intensity_limits is not None:
        cargs.extend([
            "--mm_{min,max}_{inside,outside}",
            mm_intensity_limits
        ])
    if cover_seg is not None:
        cargs.extend([
            "--cover-seg",
            cover_seg
        ])
    if first_peak_d1:
        cargs.append("--first-peak-d1")
    if first_peak_d2:
        cargs.append("--first-peak-d2")
    if white_border_low_factor is not None:
        cargs.extend([
            "--white_border_low_factor",
            str(white_border_low_factor)
        ])
    if fill_lateral_ventricles is not None:
        cargs.extend([
            "--fill-lat-vents",
            *map(str, fill_lateral_ventricles)
        ])
    cargs.append("[COST_FUNCTION_ARGS]")
    ret = MrisPlaceSurfaceOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_PLACE_SURFACE_METADATA",
    "MrisPlaceSurfaceOutputs",
    "mris_place_surface",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SURF_LOCALSTAT_METADATA = Metadata(
    id="5a51275b28d3b9824220c99fed994126246f2f36.boutiques",
    name="SurfLocalstat",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class SurfLocalstatOutputs(typing.NamedTuple):
    """
    Output object returned when calling `surf_localstat(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType
    """Computed local statistics dataset"""


def surf_localstat(
    prefix: str,
    stat_: typing.Literal["mean", "mode", "num", "FWHM", "ALL"],
    input_dataset: InputPathType,
    surface: InputPathType,
    hood: float | None = None,
    nbhd_rad: float | None = None,
    runner: Runner | None = None,
) -> SurfLocalstatOutputs:
    """
    Compute local statistics on a surface mesh.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        prefix: Prefix of output data set.
        stat_: Compute the specified statistic on the values extracted from the\
            region around each voxel. Options: mean, mode, num, FWHM, ALL.
        input_dataset: Input dataset.
        surface: Input GIFTI surface file.
        hood: Neighborhood of nodes within the specified radius R.
        nbhd_rad: Distance from node n as measured by the shortest distance\
            along the mesh.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfLocalstatOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURF_LOCALSTAT_METADATA)
    cargs = []
    cargs.append("SurfLocalstat")
    if hood is not None:
        cargs.extend([
            "-hood",
            str(hood)
        ])
    if nbhd_rad is not None:
        cargs.extend([
            "-nbhd_rad",
            str(nbhd_rad)
        ])
    cargs.extend([
        "-prefix",
        prefix
    ])
    cargs.extend([
        "-stat",
        stat_
    ])
    cargs.extend([
        "-input",
        execution.input_file(input_dataset)
    ])
    cargs.extend([
        "-i_gii",
        execution.input_file(surface)
    ])
    ret = SurfLocalstatOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file(prefix + ".niml.dset"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SURF_LOCALSTAT_METADATA",
    "SurfLocalstatOutputs",
    "surf_localstat",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3DANISOSMOOTH_METADATA = Metadata(
    id="0f940845485d0f2e6aa4a1bbafdccc58018a21a2.boutiques",
    name="3danisosmooth",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3danisosmoothOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3danisosmooth(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_dataset: OutputPathType | None
    """Output dataset after anisotropic smoothing"""
    gradient_data: OutputPathType
    """Gradient dataset saved at each iteration"""
    eigens_data: OutputPathType
    """Eigens dataset saved at each iteration"""
    phi_data: OutputPathType
    """Phi dataset saved at each iteration"""
    dtensor_data: OutputPathType
    """Dtensor dataset saved at each iteration"""
    ematrix_data: OutputPathType
    """Ematrix dataset saved at the first sub-brick iteration"""
    flux_data: OutputPathType
    """Flux dataset saved at the first sub-brick iteration"""
    gmatrix_data: OutputPathType
    """Gmatrix dataset saved at the first sub-brick iteration"""
    diff_measures_data: OutputPathType
    """Dataset containing FA, MD, Cl, Cp, and Cs values saved at each
    iteration"""


def v_3danisosmooth(
    input_dataset: InputPathType,
    prefix: str | None = None,
    iterations: float | None = None,
    v_2d_flag: bool = False,
    v_3d_flag: bool = False,
    mask_dataset: InputPathType | None = None,
    automask_flag: bool = False,
    viewer_flag: bool = False,
    nosmooth_flag: bool = False,
    sigma1: float | None = None,
    sigma2: float | None = None,
    deltat: float | None = None,
    savetempdata_flag: bool = False,
    save_temp_with_diff_measures_flag: bool = False,
    phiding_flag: bool = False,
    phiexp_flag: bool = False,
    noneg_flag: bool = False,
    setneg_value: float | None = None,
    edgefraction: float | None = None,
    datum_type: str | None = None,
    matchorig_flag: bool = False,
    help_flag: bool = False,
    runner: Runner | None = None,
) -> V3danisosmoothOutputs:
    """
    Smooths a dataset using an anisotropic smoothing technique.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        input_dataset: Input dataset to be smoothed.
        prefix: Output dataset prefix name.
        iterations: Number of iterations (default=10).
        v_2d_flag: Smooth a slice at a time (default).
        v_3d_flag: Smooth through slices.
        mask_dataset: Use specified dataset as mask to include/exclude voxels.
        automask_flag: Automatically compute mask for dataset.
        viewer_flag: Show central axial slice image every iteration.
        nosmooth_flag: Do not do intermediate smoothing of gradients.
        sigma1: Gaussian smoothing sigma before gradient computation\
            (default=0.5).
        sigma2: Gaussian smoothing sigma after gradient computation\
            (default=1.0).
        deltat: Pseudo-time step (default=0.25).
        savetempdata_flag: Save temporary datasets each iteration.
        save_temp_with_diff_measures_flag: Save temporary datasets with\
            different measures in a dataset.
        phiding_flag: Use Ding method for computing phi (default).
        phiexp_flag: Use exponential method for computing phi.
        noneg_flag: Set negative voxels to 0.
        setneg_value: Set negative voxels to specified value.
        edgefraction: Adjust the fraction of the anisotropic component added (0\
            to 1, default=0.5).
        datum_type: Specify type for output data (byte, short, float)\
            [default=float].
        matchorig_flag: Match datum type and clip min/max to input data.
        help_flag: Print help message.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3danisosmoothOutputs`).
    """
    if edgefraction is not None and not (0 <= edgefraction <= 1): 
        raise ValueError(f"'edgefraction' must be between 0 <= x <= 1 but was {edgefraction}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3DANISOSMOOTH_METADATA)
    cargs = []
    cargs.append("3danisosmooth")
    cargs.append(execution.input_file(input_dataset))
    if prefix is not None:
        cargs.extend([
            "-prefix",
            prefix
        ])
    if iterations is not None:
        cargs.extend([
            "-iters",
            str(iterations)
        ])
    if v_2d_flag:
        cargs.append("-2D")
    if v_3d_flag:
        cargs.append("-3D")
    if mask_dataset is not None:
        cargs.extend([
            "-mask",
            execution.input_file(mask_dataset)
        ])
    if automask_flag:
        cargs.append("-automask")
    if viewer_flag:
        cargs.append("-viewer")
    if nosmooth_flag:
        cargs.append("-nosmooth")
    if sigma1 is not None:
        cargs.extend([
            "-sigma1",
            str(sigma1)
        ])
    if sigma2 is not None:
        cargs.extend([
            "-sigma2",
            str(sigma2)
        ])
    if deltat is not None:
        cargs.extend([
            "-deltat",
            str(deltat)
        ])
    if savetempdata_flag:
        cargs.append("-savetempdata")
    if save_temp_with_diff_measures_flag:
        cargs.append("-save_temp_with_diff_measures")
    if phiding_flag:
        cargs.append("-phiding")
    if phiexp_flag:
        cargs.append("-phiexp")
    if noneg_flag:
        cargs.append("-noneg")
    if setneg_value is not None:
        cargs.extend([
            "-setneg",
            str(setneg_value)
        ])
    if edgefraction is not None:
        cargs.extend([
            "-edgefraction",
            str(edgefraction)
        ])
    if datum_type is not None:
        cargs.extend([
            "-datum",
            datum_type
        ])
    if matchorig_flag:
        cargs.append("-matchorig")
    if help_flag:
        cargs.append("-help")
    ret = V3danisosmoothOutputs(
        root=execution.output_file("."),
        output_dataset=execution.output_file(prefix + "+smooth") if (prefix is not None) else None,
        gradient_data=execution.output_file("Gradient.ITER"),
        eigens_data=execution.output_file("Eigens.ITER"),
        phi_data=execution.output_file("phi.ITER"),
        dtensor_data=execution.output_file("Dtensor.ITER"),
        ematrix_data=execution.output_file("Ematrix.ITER"),
        flux_data=execution.output_file("Flux.ITER"),
        gmatrix_data=execution.output_file("Gmatrix.ITER"),
        diff_measures_data=execution.output_file("Diff_measures.ITER"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3danisosmoothOutputs",
    "V_3DANISOSMOOTH_METADATA",
    "v_3danisosmooth",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

TIME_SCCAN_METADATA = Metadata(
    id="c29edfffbdbeee9e02e8801bed3cb5f939e9ea50.boutiques",
    name="TimeSCCAN",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


@dataclasses.dataclass
class TimeSccanTimeseriesimageToMatrix:
    """
    Takes a timeseries (4D) image and converts it to a 2D matrix csv format as
    output. If the mask has multiple labels (more than one), then the average
    time series in each label will be computed and put in the csv.
    """
    timeseries_image: InputPathType
    mask_image: InputPathType
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append(execution.input_file(self.timeseries_image) + execution.input_file(self.mask_image))
        return cargs


@dataclasses.dataclass
class TimeSccanNetworkScca:
    time_matrix: InputPathType
    label_matrix: InputPathType
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("scca[" + execution.input_file(self.time_matrix) + "," + execution.input_file(self.label_matrix) + "]")
        return cargs


@dataclasses.dataclass
class TimeSccanNetworkRegionAveraging:
    time_matrix: InputPathType
    label_matrix: InputPathType
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("region-averaging[" + execution.input_file(self.time_matrix) + "," + execution.input_file(self.label_matrix) + "]")
        return cargs


class TimeSccanOutputs(typing.NamedTuple):
    """
    Output object returned when calling `time_sccan(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    correlation_matrix: OutputPathType
    """The output is the 2D correlation matrix."""


def time_sccan(
    output: InputPathType,
    number_consecutive_labels: int | None = 0,
    minimum_region_size: int | None = 1,
    iterations: int | None = 20,
    sparsity: float | None = 0.1,
    n_eigenvectors: int | None = 2,
    robustify: int | None = 0,
    l1: float | None = 0,
    cluster_thresh: int | None = 1,
    ridge_cca: int | None = 0,
    partial_scca_option: typing.Literal["PQ", "PminusRQ", "PQminusR", "PminusRQminusR"] | None = None,
    timeseriesimage_to_matrix: TimeSccanTimeseriesimageToMatrix | None = None,
    labelsimage_to_matrix: InputPathType | None = None,
    network: typing.Union[TimeSccanNetworkScca, TimeSccanNetworkRegionAveraging] | None = None,
    runner: Runner | None = None,
) -> TimeSccanOutputs:
    """
    A tool for sparse statistical analysis on connectivity within a subject.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        output: Output is a 2D correlation matrix.
        number_consecutive_labels: Number of consecutive labels in data.
        minimum_region_size: Minimum size of a region: regions below this size\
            are given a 0.0 connectivity value.
        iterations: Number of iterations.
        sparsity: Sparsity - a float from (0,1] indicating what fraction of the\
            data to use.
        n_eigenvectors: Number of permutations to use in scca.
        robustify: Rank-based scca.
        l1: Use l1 ( > 0 ) or l0 ( < 0 ) penalty, also sets gradient step size\
            e.g. -l 0.5 ( L1 ) , -l -0.5 (L0) will set 0.5 grad descent step for\
            either penalty.
        cluster_thresh: Cluster threshold on view P.
        ridge_cca: Number of permutations to use in scca.
        partial_scca_option: Choices for partial SCCA: PQ, PminusRQ, PQminusR,\
            PminusRQminusR.
        timeseriesimage_to_matrix: Takes a timeseries (4D) image and converts\
            it to a 2D matrix csv format as output. If the mask has multiple labels\
            (more than one), then the average time series in each label will be\
            computed and put in the csv.
        labelsimage_to_matrix: Takes a labeled (3D) image and converts it to a\
            2D matrix csv format as output.
        network: Build the network connectivity matrix.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TimeSccanOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(TIME_SCCAN_METADATA)
    cargs = []
    cargs.append("TimeSCCAN")
    cargs.extend([
        "--output",
        execution.input_file(output)
    ])
    if number_consecutive_labels is not None:
        cargs.extend([
            "-l",
            str(number_consecutive_labels)
        ])
    if minimum_region_size is not None:
        cargs.extend([
            "-R",
            str(minimum_region_size)
        ])
    if iterations is not None:
        cargs.extend([
            "-i",
            str(iterations)
        ])
    if sparsity is not None:
        cargs.extend([
            "-s",
            str(sparsity)
        ])
    if n_eigenvectors is not None:
        cargs.extend([
            "-n",
            str(n_eigenvectors)
        ])
    if robustify is not None:
        cargs.extend([
            "-r",
            str(robustify)
        ])
    if l1 is not None:
        cargs.extend([
            "-l",
            str(l1)
        ])
    if cluster_thresh is not None:
        cargs.extend([
            "--ClusterThresh",
            str(cluster_thresh)
        ])
    if ridge_cca is not None:
        cargs.extend([
            "-e",
            str(ridge_cca)
        ])
    if partial_scca_option is not None:
        cargs.extend([
            "--partial-scca-option",
            partial_scca_option
        ])
    if timeseriesimage_to_matrix is not None:
        cargs.extend([
            "--timeseriesimage-to-matrix",
            *timeseriesimage_to_matrix.run(execution)
        ])
    if labelsimage_to_matrix is not None:
        cargs.extend([
            "--labelsimage-to-matrix",
            execution.input_file(labelsimage_to_matrix)
        ])
    if network is not None:
        cargs.extend([
            "--network",
            *network.run(execution)
        ])
    ret = TimeSccanOutputs(
        root=execution.output_file("."),
        correlation_matrix=execution.output_file(pathlib.Path(output).name),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "TIME_SCCAN_METADATA",
    "TimeSccanNetworkRegionAveraging",
    "TimeSccanNetworkScca",
    "TimeSccanOutputs",
    "TimeSccanTimeseriesimageToMatrix",
    "time_sccan",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_EULER_NUMBER_METADATA = Metadata(
    id="0ad5734919ffe0fbdc78eddad073fdf8082b2ef3.boutiques",
    name="mris_euler_number",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisEulerNumberOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_euler_number(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    outfile: OutputPathType | None
    """File where the number of holes is written"""


def mris_euler_number(
    input_surface: InputPathType,
    output_file: str | None = None,
    runner: Runner | None = None,
) -> MrisEulerNumberOutputs:
    """
    This program computes EulerNumber for a cortical surface.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_surface: Input surface file.
        output_file: Write number of holes to output file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisEulerNumberOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_EULER_NUMBER_METADATA)
    cargs = []
    cargs.append("mris_euler_number")
    cargs.append(execution.input_file(input_surface))
    if output_file is not None:
        cargs.extend([
            "-o",
            output_file
        ])
    ret = MrisEulerNumberOutputs(
        root=execution.output_file("."),
        outfile=execution.output_file(output_file) if (output_file is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_EULER_NUMBER_METADATA",
    "MrisEulerNumberOutputs",
    "mris_euler_number",
]
